;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; GDT
;-----------------------------------------------------------------------------------

align 8

NULL_SELECTOR         = 0
DATA_SELECTOR         = 1 shl 3 				; flat data selector (ring 0)
CODE_SELECTOR         = 2 shl 3 				; 32-bit code selector (ring 0)
LONG_SELECTOR         = 3 shl 3 				; 64-bit code selector (ring 0)
COMPABILITY_SELECTOR  = 4 shl 3 				; 64-bit code selector (ring 0)

GDTR:								; Global Descriptors Table Register
  dw 5*8-1							; limit of GDT (size minus one)
  dq GDT							; linear address of GDT

GDT 		dw 0x0000,0x0000,0x0000,0x0000			; Null descriptor
    		dw 0FFFFh,0,9200h,08Fh				; flat data descriptor
    		dw 0FFFFh,0,9A00h,0CFh				; 32-bit code descriptor
    		dw 0FFFFh,0,9A00h,0AFh				; 64-bit code descriptor
COMP_DESC:	dw 0FFFFh,0,9A00h,0CFh				; 64-bit code descriptor (compability mode)

;     bit63<--------------------------------------------------------------------->bit0
; code64 = 00000000 10101111 10011010 00000000 00000000 00000000 11111111 11111111
; data64 = 00000000 10001111 10010010 00000000 00000000 00000000 11111111 11111111


; ===================================================================================================
; EOF


; Descriptors
;
;
; Code Segment Descriptor (legacy mode)
;------------------------
; 
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;  -----------------------------------------------------------------------------------------------
; |                       |  |  |  |A |  Segment  |  |     |  |  |  |  |  |                       |
; | Base Address 31-24    |G |D |  |V |  Limit    |P | DPL |1 |1 |C |R |A |  Base Address 23-16   | +4
; |                       |  |  |  |L |  19-16    |  |     |  |  |  |  |  |                       |
;  -----------------------------------------------------------------------------------------------
; |                                               |                                               |
; |           Base Address 15-0                   |           Segment Limit 15-0                  | +0
; |                                               |                                               |
;  -----------------------------------------------------------------------------------------------
;
;
; Data Segment Descriptor (legacy mode)
;------------------------
; 
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;  -----------------------------------------------------------------------------------------------
; |                       |  |D |  |A |  Segment  |  |     |  |  |  |  |  |                       |
; | Base Address 31-24    |G |/ |  |V |  Limit    |P | DPL |1 |0 |E |W |A |  Base Address 23-16   | +4
; |                       |  |B |  |L |  19-16    |  |     |  |  |  |  |  |                       |
;  -----------------------------------------------------------------------------------------------
; |                                               |                                               |
; |           Base Address 15-0                   |           Segment Limit 15-0                  | +0
; |                                               |                                               |
;  -----------------------------------------------------------------------------------------------
;
;
;-----------
; Code
;-----------
; Bit 11 set to 1     = code-segment descriptors
; (Bit 11 cleared to 0 = data-segment descriptors)
;
; S bit (12) set to 1 = user segments.
;
; Conforming (C) Bit. Bit 10 of byte +4. Set to 1 = code segment as conforming.
; When control is transferred to a higher-privilege conforming code-segment (C=1) from a lower
; privilege code segment, the processor CPL does not change.
; (If 1, code in this segment can be executed from an equal or lower privilege level.)
;
; Readable (R) Bit. Bit 9 of byte +4. Set to 1 = code segment is both executable
; and readable as data. Set to 0, the code segment is executable.
;
; Accessed (A) Bit. Bit 8 of byte +4. Set to 1 by the processor when the descriptor is
; copied from the GDT or LDT into the CS register.

; Code-Segment Default-Operand Size (D) Bit. Bit 22 of byte +4.
; Selects the default operand size and address sizes. In legacy mode,
; when D=0 operand size and address size = 16 bits
; when D=1 operand size and address size = 32 bits.
;
;----------
; Data
;----------
; (Bit 11 set to 1     = code-segment descriptors)
; Bit 11 cleared to 0 = data-segment descriptors
; 
; Expand-Down (E) Bit. Bit 10 of byte +4. Set to 1 = data segment as expand-down.
; Useful for stacks. Clearing to 0 = the data segment as expand-up.
;
; Writable (W) Bit. Bit 9 of byte +4. Sett to 1 = data segment as read/write.
; Cleared to 0 = the segment is read-only.
; 
; Data-Segment Default Operand Size (D/B) Bit. Bit 22 of byte +4.
; For expand-down data segments (E=1):
; D=1 sets the upper bound of the segment at 0_FFFF_FFFFh.
; D=0 sets the upper bound of the segment at 0_FFFFh.
; If data segment is referenced by the stack selector (SS), the D bit is referred to as the B bit.
; The B bit sets the default stack size.
; B=1 => 32-bit stack referenced by the 32-bit ESP register.
; B=0 => 16-bit stack referenced by the 16-bit SP register.

; P  : Present bit. This must be 1 for all valid selectors.
; DPL: Privilege, 2 bits. Contains the ring level, 0 = highest (kernel), 3 = lowest (user applications).
; G  : Granularity bit. If 0 the limit is in 1 Byte blocks (byte granularity),
;                       If 1 the limit is in 4 KiB blocks (page granularity).
; D  : Size bit. Set to 0 = 16 bit protected mode. Set to 1 = 32 bit protected mode.


; Code Segment Descriptor (long mode)
;------------------------
; 
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;  -----------------------------------------------------------------------------------------------
; |                       |  |  |  |  |           |  |     |  |  |  |  |  |                       |
; |                       |  |D |L |  |           |P | DPL |  |  |C |  |  |                       | +4
; |                       |  |  |  |  |           |  |     |  |  |  |  |  |                       |
;  -----------------------------------------------------------------------------------------------
; |                                               |                                               |
; |                                               |                                               | +0
; |                                               |                                               |
;  -----------------------------------------------------------------------------------------------
;
; Data Segment Descriptor (long mode)
;------------------------
; 
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;  -----------------------------------------------------------------------------------------------
; |                       |  |  |  |  |           |  |     |  |  |  |  |  |                       |
; |                       |  |  |  |  |           |P |     |  |  |  |  |  |                       | +4
; |                       |  |  |  |  |           |  |     |  |  |  |  |  |                       |
;  -----------------------------------------------------------------------------------------------
; |                                               |                                               |
; |                                               |                                               | +0
; |                                               |                                               |
;  -----------------------------------------------------------------------------------------------
;
;
; Fields Ignored in 64-Bit Mode.
; Segmentation is disabled. Code-segment base addresses are ignored, treated as if it has a value of zero.
; Segment-limit field, G, R, A, D/B, E, W, DPL are ignored.
; Segment-limit checking and segment-privilege checks are not performed.
; The virtual address is checked. (to see if it is in canonical-address form)

; Long (L) Attribute Bit. Bit 21 of byte +4. NEW. (In legacy mode this bit is reserved.)
; L=1 the processor is running in 64-bit mode
; L=0 the processor is running in compatibility mode.
;
; With L=0 in code-segment you can execute existing 16-bit and 32-bit applications and
; the legacy D bit (address-size, operand-size) is observed.
;
; With L=1 the only valid setting of the D bit is 0.
; The operand size = 32 bits, address size = 64 bits.
; These default sizes can be overridden (with operand size, address size, REX prefixes).
; (L=1 and D=1) is reserved for future use.




; Selector:
;-----------------------------
; The selector format's fields:
;
; Selector Index Field. Bits 15-3.
; The selector-index field specifies an entry in the descriptor table.
; Each entries are eight bytes long.
; Descriptor-table base address + selector index scaled by 8 (offset) => descriptor-entry
; address in virtual-address space.
; 
; Table Indicator (TI) Bit. Bit 2.
; Descriptor table used.
; TI=0 => is used the GDT
; TI=1 => is used the LDT

; Requestor Privilege-Level (RPL) Field. Bits 1-0.
; RPL = privilege level (CPL) of processor when the selector is created.
;
; Null Selector.
; (Selector index=0 and TI=0) First entry in the GDT used to
; invalidate unused segment registers.


