;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; VESA
;
; Based on long mode examples by Mac2004
; available at FASM board.
;-----------------------------------------------------------------------------------


	align 4
	LFB_start  	 dd 0		; save LFB pointers here
	LFB_width  	 dw 0		;
	LFB_height	 dw 0		;
	GUI     	 db 0		; 0=Text mode 1=GUI

;---------------------------------------------------
;  {VESA VBE modeInfoBlock}
;---------------------------------------------------


struc VBE_MODEINFOBLOCK_STRUCTURE 
	{

	;general fields!
	;---------------

      	.modeattr 	dw 0    	;mode attributes
      	.wina  		db 0     	;window A attributes
      	.winb  		db 0     	;window B attributes
      	.gran  		dw 0     	;window granularity in K
      	.winsiz 	dw 0     	;window size in K
      	.sega  		dw 0     	;start segment of window A
      	.segb  		dw 0     	;start segment of window B        
      	.pagefunc 	dd 0    	;FAR window positioning function (equivalent to AX=4F05h)
      	.bytesperline 	dw 0		;bytes per scan line

        ;---remainder is optional for VESA modes, needed for OEM modes---

       .width 		dw 0   		; width in pixels
       .height 		dw 0  		; height in pixels
       .charw 		db 0   		; width of character cell in pixels
       .charh 		db 0   		; height of character cell in pixels
       .planes 		db 0   		; number of memory planes
       .bits  		db 0      	; number of bits per pixel
       .nbanks 		db 0     	; number of banks
       .model 		db 0      	; memory model type
       .banks 		db 0      	; size of bank in K
       .images 		db 0     	; number of image pages
       .resv   		db 0        	; reserved(1)
        ;------VBE v1.2+ ---------------------------------------------------------------- 
       .redinf 		dw 0     	;{red   - low byte = #bits, high byte = start pos}
       .grninf 		dw 0     	;{green - low byte = #bits, high byte = start pos}
       .bluinf 		dw 0     	;{blue  - low byte = #bits, high byte = start pos}
       .resinf 		dw 0       	; reserved(2)
       .info   		db 0     	;{Bit 0: Gamma, 1: d field}


        ;------VBE v2+ ------------------------------------------------------------------- 
       .phys_flat_adr 	dd 0       	;Physical addds of linear video buffer
       .offscr 		dd 0     	;Pointer to start of offscreen memory
       .offsiz 		dw 0     	;{Size of offscreen memory in Kb}

	;-- Mandatory information for VBE 3.0 and above-----------------------------------

        .BytesPerLine 	dw 0      	;bytes per scan line for linear modes
        .BankedPages 	db 0       	;number of images for banked modes
        .LinearPages 	db 0       	;number of images for linear modes
        .RedMaskSize 	db 0       	;size of direct color red mask (linear modes)
        .RedFieldPos 	db 0       	;bit position of lsb of red mask (linear modes)
        .GreenMaskSize 	db 0     	;size of direct color green mask (linear modes)
        .GreenFieldPos 	db 0     	;bit position of lsb of green mask (linear modes)
        .BlueMaskSize 	db 0      	;size of direct color blue mask (linear modes)
        .BlueFieldPos 	db 0      	;bit position of lsb of blue mask (linear modes)
        .RsvdMaskSize 	db 0      	;size of direct color derved mask (linear modes)
        .RsvdFieldPos 	db 0      	;bit position of lsb of derved mask (linear modes)
        .MaxPixelClock 	dd 0     	;maximum pixel clock (in Hz) for graphics mode
        .OEMBuffer: 	times 189 db 0  ;OEM scratchpad

	.struc_size	dw .struc_size-.modeattr
	};end of structure

;-------------------------------------------------------------------
;  {VESA VBE (VBE_INFO_BLOCK}
;-------------------------------------------------------------------

struc VBE_INFO_BLOCK_STRUCTURE 
	{
	;general fields!
	;---------------

        .sign 		dd 0      	;db 4    ;{Must be 'VESA'}
        .vers 		dw 0      	;{VBE version number.}
        .oemadr 	dd 0    	;pointer to OEM name
        .capab 		dd 0     	;db 4    ;capabilities.
        .modelist 	dd 0  		;pointer to list of supported VESA and
                       			;OEM video modes Terminated with 0FFFFh.
        .memory  	dw 0   		;Video memory in 64k blocks

	;Remaining fields are for VBE 2.0 or later
	;------------------------------------------

        .oemver 	dw 0   		;{OEM version}
        .vendor 	dd 0   		;{Vendor name}
        .product 	dd 0  		;{Product Name}
        .prodrev 	dd 0    	;{Product Revision}

        .OEMBuffer: 	times 512 db 0 	;OEM scratchpad ;DEBUG!!

	};end of structure


         Vesa_info 		VBE_INFO_BLOCK_STRUCTURE        ;new struc for VBE info block.
         ModeInfoBlockPtr 	VBE_MODEINFOBLOCK_STRUCTURE	;Structure for obtaining specific mode info.(system structure!)


;------------------------------------------------------------



;-------------------------------------------------------------------
; RM_Check_VBE  -   SEE IF THERE'S A VESA BIOS  PRESENT
;
; input:  --
; output: --
;-------------------------------------------------------------------

RM_Check_VBE:

	push es
	pusha

	;SEE IF THERE'S A VESA BIOS
	;---------------------------

	;-----------------------------------------------------------
	;Round 1... Vesa_info.sign needs to contain 'VESA'=$32454256
	;after the bios call.
	;-----------------------------------------------------------
	push        es
	mov	    ax,4f00h			; get VBE controller information.
	mov	    di,Vesa_info		; use our struc to save the info.
	push        cs
	pop	    es
	int	    10h				; do it!
	pop	    es

	cmp	    ax,004fh			; test result.
	je	    .VBEFound

	mov ax,200+7				;start point
	mov ch,' '				;color
	mov bx,vbe_not_ok_msg			;str
	call RM_PutStr

	jmp .continue
.VBEFound:

	mov ax,200+7				;start point
	mov ch,' '				;color
	mov bx,vbe_ok_msg			;str
	call RM_PutStr

.continue:
	; let's save "VBE2" to see, if above vesa2.0
	;----------------------------------------------
	mov eax,032454256h			; don't remove, will crash totally!!
	mov dword[Vesa_info.sign],eax		; "VBE2" - enable VBE2 if present

	mov ax, $4f00				; 4F00h --GET SuperVGA INFORMATION
	mov di, Vesa_info			; get buffer pointer
	int $10					; do it

	cmp ax, 004fh				; if ok with the query ?
	jne .vbe2_fail				; not ok with vbe2+!


	mov eax,41534556h               	; test for vbe2 result.
	cmp dword[Vesa_info.sign],eax		; "VBE2" - enable VBE2 if present
	jne .vbe2_fail

	;Here's everything ok!!!

	mov ax,200+80+7				;start point
	mov ch,' '				;color
	mov bx,vbe2_ok_msg			;str
	call RM_PutStr
	jmp .vbe2ok

	;only old vbe1+ card was located ;--> let's tell the user.
	;----------------------------------------------------------

.vbe2_fail:
	mov ax,200+80+7				;start point
	mov ch,' '				;color
	mov bx,vbe2_not_ok_msg			;str
	call RM_PutStr

	;Ask info on requested vesa mode
	;-------------------------------
.vbe2ok:

.done:
	popa
	pop es
	ret       				; end of procedure


;-------------------------------------------------------------------
; RM_Init_VBE - Initialize LFB mode
;
; input:  --
; output: --
;-------------------------------------------------------------------
RM_Init_VBE:

	push es
	pusha

	;init selected LFB-mode here!
	;----------------------------

	xor  ebx,ebx
	mov  bx,0x111					;640x480x16bit
	or   ebx,04000h					;Toggle lfb flag on
	mov  ax, $4f02					;init vesamode fuction
	int  $10

	;Ask info on requested vesa mode
	;-------------------------------

	mov cx,0x111					;get mode info structure
	mov di,ModeInfoBlockPtr				;pointer to mode info block
	mov ax,4F01h					; Get VBE Mode Information
	int 10h						; Do it!

	mov ebx, [ModeInfoBlockPtr.phys_flat_adr]	;get LFB pointer
	mov [LFB_start], ebx				;save for later use
	mov bx, [ModeInfoBlockPtr.width]		;get LFB X res
	mov [LFB_width], bx				;save for later use
	mov bx, [ModeInfoBlockPtr.height]		;get LFB Y res
	mov [LFB_height], bx				;save for later use

.done:
	popa
	pop es
	ret       					; end of procedure

; ===================================================================================================
; EOF

