;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
;
; Init SMP
;-----------------------------------------------------------------------------------


LM_Init_SMP:

	sti				; Enable interrupts (the timer)

	mov al, '3'			; Start of MP init
	mov [0x000B809C], al
	mov al, '0'
	mov [0x000B809E], al

; Step 1: Get APIC Information via ACPI
smp_check_for_acpi:			; Look for the Root System Description Pointer Structure
	mov rsi, 0x00000000000E0000	; We want to start looking here
	mov rbx, 'RSD PTR '		; This in the Signature for the ACPI Structure Table (0x2052545020445352)
searchingforACPI:
	lodsq				; Load a quad word from RSI and store in RAX, then increment RSI by 8
	cmp rax, rbx
	je foundACPI
	cmp rsi, 0x00000000000FFFFF	; Keep looking until we get here
	jae noMP			; We can't find ACPI either.. bail out and default to single cpu mode
	jmp searchingforACPI 

	mov al, '3'			; ACPI tables detected
	mov [0x000B809C], al
	mov al, '2'
	mov [0x000B809E], al

foundACPI:
;-----------------------------------------
	jmp init_smp_acpi 
;-----------------------------------------

makempgonow:
	mov al, '3'			; ACPI tables parsed
	mov [0x000B809C], al
	mov al, '6'
	mov [0x000B809E], al



; Step 2: Enable Local APIC on BSP
	mov rsi, [os_LocalAPICAddress]
	cmp rsi, 0x00000000
	je noMP				; Skip MP init if we didn't get a valid LAPIC address
	add rsi, 0xf0			; Offset to Spurious Interrupt Register
	mov rdi, rsi
	lodsd
	or eax, 0000000100000000b
	stosd


; Step 3: Start the AP's one by one
	xor eax, eax
	xor ecx, ecx
	xor edx, edx
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x20			; Add the offset for the APIC ID location
	lodsd				; APIC ID is stored in bits 31:24
	shr rax, 24			; AL now holds the BSP CPU's APIC ID
	mov dl, al			; Store BSP APIC ID in DL
	mov rsi, 0x0000000000005800
	xor eax, eax

nextcore:
	cmp rsi, 0x0000000000005900
	je done
	lodsb
	cmp al, 1			; Is it enabled?
	jne skipcore


	cmp cl, dl			; Is it the BSP?  cl = APIC ID
	je skipcore



; Broadcast 'INIT' IPI to APIC ID in AL
	mov al, cl
	shl eax, 24
	mov rdi, [os_LocalAPICAddress]
	add rdi, 0x310
	stosd
	mov eax, 0x00004500
	mov rdi, [os_LocalAPICAddress]
	add rdi, 0x300
	stosd
	push rsi
verifyinit:
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x300
	lodsd
	bt eax, 12			; Verify that the command completed
	jc verifyinit
	pop rsi

;----------------------------
	mov rax, [os_Counter]
	add rax, 10
wait1:
	mov rbx, [os_Counter]
	cmp rax, rbx
	ja wait1
;----------------------------



; Broadcast 'Startup' IPI to destination using vector 0x08 to specify entry-point is at the memory-address 0x00008000
	mov al, cl
	shl eax, 24
	mov rdi, [os_LocalAPICAddress]
	add rdi, 0x310
	stosd
	mov eax, 0x00004608		; Vector 0x08
	mov rdi, [os_LocalAPICAddress]
	add rdi, 0x300
	stosd
	push rsi
verifystartup1:
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x300
	lodsd
	bt eax, 12			; Verify that the command completed
	jc verifystartup1
	pop rsi

;----------------------------
	mov rax, [os_Counter]
	add rax, 2
wait2:
	mov rbx, [os_Counter]
	cmp rax, rbx
	ja wait2
;----------------------------



skipcore:
	inc cl
	jmp nextcore




done:
	mov al, '3'
	mov [0x000B809C], al
	mov al, '8'
	mov [0x000B809E], al

; Let things settle (Give the AP's some time to finish)
;---------------------------------
	mov rax, [os_Counter]
	add rax, 10
wait3:
	mov rbx, [os_Counter]
	cmp rax, rbx
	ja wait3
;---------------------------------




; Step 4: Finish up

noMP:
	lock
	inc word [cpu_activated]	; BSP adds one here

	xor eax, eax
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x20			; Add the offset for the APIC ID location
	lodsd				; APIC ID is stored in bits 31:24
	shr rax, 24			; AL now holds the CPU's APIC ID (0 - 255)
	mov rdi, 0x0000000000005700	; The location where the cpu values are stored
	add rdi, rax			; RDI points to infomap CPU area + APIC ID. ex 5701 would be APIC ID 1
	mov al, 3			; This is the BSP so bits 0 and 1 are set
	stosb

	mov al, '3'
	mov [0x000B809C], al
	mov al, 'A'
	mov [0x000B809E], al


	cli				; Disable interrupts (the timer)


	; Reset the stack to the proper location (was set to 0x8000 previously)
	cli
	mov rsi, [os_LocalAPICAddress]	; We would call os_smp_get_id here but the stack is not ...
	add rsi, 0x20			; ... yet defined. It is safer to find the value directly.
	lodsd				; Load a 32-bit value. We only want the high 8 bits
	shr rax, 24			; Shift to the right and AL now holds the CPU's APIC ID
	shl rax, 12			; shift left 12 bits for a 4096 byte stack
	add rax, 0x0000000000201000	; stacks decrement when you "push", start at 4096 bytes in
	mov rsp, rax			; (AP's stacks reset in trampoline.inc)
	sti


	; Debug
	mov al, '3'			; SMP Init complete
	mov [0x000B809C], al
	mov al, 'E'
	mov [0x000B809E], al


	jmp smp_ended


; END
; =============================================================================




;
; INIT SMP ACPI
; =============================================================================


init_smp_acpi:
	lodsb				; Checksum
	lodsd				; OEMID (First 4 bytes)
	lodsw				; OEMID (Last 2 bytes)
	lodsb				; Grab the Revision value (0 is v1.0, 1 is v2.0, 2 is v3.0, etc)
	add al, 49
	mov [0x000B8098], al		; Print the ACPI version number
	sub al, 49
	cmp al, 0
	je foundACPIv1			; If AL is 0 then the system is using ACPI v1.0
	jmp foundACPIv2			; Otherwise it is v2.0 or higher

foundACPIv1:
	xor eax, eax
	lodsd				; Grab the 32 bit physical address of the RSDT (Offset 16).
	mov rsi, rax			; RSI now points to the RSDT
	lodsd				; Grab the Signiture
	cmp eax, 'RSDT'			; Make sure the signiture is valid
	jne novalidacpi			; Not the same? Bail out
	sub rsi, 4
	mov [os_ACPITableAddress], rsi	; Save the RSDT Table Address
	add rsi, 4
	xor eax, eax
	lodsd				; Length
	add rsi, 28			; Skip to the Entry offset
	sub eax, 36			; EAX holds the table size. Subtract the preamble
	shr eax, 2			; Divide by 4
	mov rdx, rax			; RDX is the entry count
	xor ecx, ecx
foundACPIv1_nextentry:
	lodsd
	push rax
	add ecx, 1
	cmp ecx, edx
	je findAPICTable
	jmp foundACPIv1_nextentry

foundACPIv2:
	lodsd				; RSDT Address
	lodsd				; Length
	lodsq				; Grab the 64 bit physical address of the XSDT (Offset 24).
	mov rsi, rax			; RSI now points to the XSDT
	lodsd				; Grab the Signiture
	cmp eax, 'XSDT'			; Make sure the signiture is valid
	jne novalidacpi			; Not the same? Bail out
	sub rsi, 4
	mov [os_ACPITableAddress], rsi	; Save the XSDT Table Address
	add rsi, 4
	xor eax, eax
	lodsd				; Length
	add rsi, 28			; Skip to the start of the Entries (offset 36)
	sub eax, 36			; EAX holds the table size. Subtract the preamble
	shr eax, 3			; Divide by 8
	mov rdx, rax			; RDX is the entry count
	xor ecx, ecx
foundACPIv2_nextentry:
	lodsq
	push rax
	add ecx, 1
	cmp ecx, edx
	jne foundACPIv2_nextentry

findAPICTable:
	mov al, '3'			; Search for the APIC table
	mov [0x000B809C], al
	mov al, '4'
	mov [0x000B809E], al
	mov ebx, 'APIC'
	xor ecx, ecx
searchingforAPIC:
	pop rsi
	lodsd
	add ecx, 1
	cmp eax, ebx
	je foundAPICTable
	cmp ecx, edx
	jne searchingforAPIC
	jmp noMP

fixstack:
	pop rax
	add ecx, 1

foundAPICTable:
	; fix the stack
	cmp ecx, edx
	jne fixstack

	lodsd				; Length of MADT in bytes
	mov ecx, eax
	xor ebx, ebx
	lodsb				; Revision
	lodsb				; Checksum

	lodsd				; OEMID (First 4 bytes)
	lodsw				; OEMID (Last 2 bytes)
	lodsq				; OEM Table ID
	lodsd				; OEM Revision
	lodsd				; Creator ID
	lodsd				; Creator Revision
	xor eax, eax
	lodsd				; Local APIC Address
	mov [os_LocalAPICAddress], rax	; Save the Address of the Local APIC
	lodsd				; Flags
	add ebx, 44
	mov rdi, 0x0000000000005800

readAPICstructures:
	cmp ebx, ecx
	jae init_smp_acpi_done
	lodsb				; APIC Structure Type


	cmp al, 0
	je APICcpu
	cmp al, 1
	je APICioapic
	jmp APICignore

APICcpu:
	inc word [cpu_detected]
	xor eax, eax
	lodsb				; Length (will be set to 8)
	add ebx, eax
	lodsb				; ACPI Processor ID

	lodsb				; APIC ID
	push rdi
	add rdi, rax
	lodsd				; Flags
	stosb
	pop rdi
	jmp readAPICstructures		; Read the next structure

APICioapic:
	xor eax, eax
	lodsb				; Length (will be set to 12)
	add ebx, eax
	lodsb				; IO APIC ID
	lodsb				; Reserved
	xor eax, eax
	lodsd				; IO APIC Address
	mov [os_IOAPICAddress], rax
	lodsd				; System Vector Base
	jmp readAPICstructures		; Read the next structure

APICignore:
	xor eax, eax
	lodsb				; We have a type that we ignore, read the next byte
	add ebx, eax
	add rsi, rax
	sub rsi, 2			; For the two bytes just read
	jmp readAPICstructures		; Read the next structure

init_smp_acpi_done:
	jmp makempgonow

novalidacpi:
	mov al, 'X'
	mov [0x000B809A], al	
	jmp $


; =============================================================================
; EOF
