;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; File System
;-----------------------------------------------------------------------------------

align 16

file_ptr  	dq 0
file_ptr_off	dq 0

tlba_temp 	dq 0

file_type	dq 0
file_size	dq 0
ncluster_d	dq 0
ncluster_p	dq 0
freespace	dq 0
npages		dq 0
mem_pages	dq 0
mem_pages_off	dq 0
mem_pages_p	dq 0
first_lba_file	dq 0
last_lba_file	dq 0
bytes_last	dq 0
lba_directory	dq 0

new_item	dq 0
level		dq 0

; fs_command
c_idx		db 0
n_args		db 0
c_string	dq 0
df_size		dq 0
pm_file		dq 0
sto_file	dq 0
sto_size	dq 0

lim_string	db 'LIM', 0
sto_string	db 'STO', 0


;---------------------------------------------------------------------------
; IN: 	rsi = command arguments string (breaked)
;	rcx = num. of words in the string
; 	rax = ptr to file in memory	(STO)
;	rbx = size			(STO)
; OUT:	rax = ptr to file in memory	(LIM)
;	rbx = size			(LIM)
;	set carry
fs_command:
	push rcx
	push rdx
	push rsi
	push rdi
	push r8
	push r9
	push r10
	push r11

	mov [sto_file], rax		; Save values for
	mov [sto_size], rbx		; STO only

	mov [n_args], cl		; Save values
	mov [c_string], rsi		; Point to first word
	call os_string_uppercase	; Convert to uppercase for comparison

	; Select device
	;-------------------------
	xor rax, rax
	mov al, 1			; device
	call os_get_strv		; in rsi

	mov eax, dword[rsi]
	call current_device
	jc .no_arg

	cmp [current.fs], 0
	je .nofs

	; Identify command
	;-------------------------------
	mov rsi, [c_string]		; Point to first word

	mov rdi, dir_string
	call os_string_compare
	jc .DIR_ARG

	mov rdi, mkd_string
	call os_string_compare
	jc .MKD_ARG

	mov rdi, mkf_string
	call os_string_compare
	jc .MKF_ARG

	mov rdi, ren_string
	call os_string_compare
	jc .REN_ARG

	mov rdi, del_string
	call os_string_compare
	jc .DEL_ARG

	;------------------

	mov rdi, lim_string
	call os_string_compare
	jc .LIM_ARG

	mov rdi, sto_string
	call os_string_compare
	jc .STO_ARG

	mov rdi, run_string
	call os_string_compare
	jc .RUN_ARG

	;------------------


	jmp .no_arg

	; Check the number of the arguments
	;----------------------------------
.DIR_ARG:
	mov [c_idx], 1
	xor rax, rax
	mov al, [n_args]		; 2-5
	cmp al, 1
	je .no_arg
	cmp al, 6
	jae .no_arg	

	sub rax, 2
	mov [level], rax		; 0-3
	jmp .level

.MKD_ARG:
	mov [c_idx], 2
	xor rax, rax
	mov al, [n_args]		; 4-5
	cmp al, 1
	je .no_arg
	cmp al, 2
	je .root
	cmp al, 3
	je .no_arg
	cmp al, 6
	jae .no_arg	

	sub rax, 3
	mov [level], rax		; 1-2
	mov [df_size], 508*4096		; new size (empty, only the header)
	jmp .level

.MKF_ARG:
	mov [c_idx], 3
	xor rax, rax
	mov al, [n_args]		; 6
	cmp al, 6
	jne .no_arg

	sub rax, 3
	mov [level], rax		; 3
	mov [df_size], 1		; new size (empty, only EOF char)
	mov [pm_file], EOF		; memory address of EOF
	jmp .level

.REN_ARG:
	mov [c_idx], 4
	xor rax, rax
	mov al, [n_args]		; 5-7

	cmp al, 3
	jbe .no_arg
	cmp al, 4
	je .root
	cmp al, 8
	jae .no_arg	

	sub rax, 4
	mov [level], rax		; 1-3
	jmp .level

.DEL_ARG:
	mov [c_idx], 5
	xor rax, rax
	mov al, [n_args]		; 4-6
	cmp al, 1
	je .no_arg
	cmp al, 2
	je .no_arg
	cmp al, 3
	je .root
	cmp al, 7
	jae .no_arg

	sub rax, 3
	mov [level], rax		; 1-3
	jmp .level

; Inner commands
;---------------------------------------
.LIM_ARG:
	mov [c_idx], 6
	xor rax, rax
	mov al, [n_args]		; 6
	cmp al, 6
	jne .no_arg

	sub rax, 3
	mov [level], rax		; 3
	jmp .level

.STO_ARG:
	mov [c_idx], 7
	xor rax, rax

	mov al, [n_args]		; 6
	cmp al, 6
	jne .no_arg

	sub rax, 3
	mov [level], rax		; 3
	jmp .level

;---------------------------------------

.RUN_ARG:
	mov [c_idx], 8
	xor rax, rax
	mov al, [n_args]		; 6
	cmp al, 6
	jne .no_arg

	sub rax, 3
	mov [level], rax		; 3
	jmp .level

;---------------------------------------


	; Load levels
	;-------------------------------
.level:
	mov rax, 0
	call fs_load_level_n
	jc .not_dir_file

	cmp [level], 0
	je .command

	mov rax, 1
	call fs_load_level_n
	jc .not_dir_file

	cmp [level], 1
	je .command

	mov rax, 2
	call fs_load_level_n
	jc .not_dir_file

	cmp [level], 2
	je .command

	mov rax, 3
	call fs_load_level_n
	jc .not_dir_file

	cmp [level], 3
	je .command

	jmp .no_arg			; Just in case

	; Command switch
	;-------------------------------
.command:
	; save values from fs_load_level_n
	mov r8, rcx			; num. of directories
	mov r9, rsi			; pointer to memory
	mov r10, rax			; pointer to (first) cluster of pointers
	mov r11, rbx			; pointer to (last) cluster of pointers

	cmp [c_idx], 1
	je .print
	cmp [c_idx], 2
	je .make
	cmp [c_idx], 3
	je .make
	cmp [c_idx], 4
	je .replace
	cmp [c_idx], 5
	je .delete
	cmp [c_idx], 6
	je .load_in_memory
	cmp [c_idx], 7
	je .store
	cmp [c_idx], 8
	je .run

	; DIR
;---------------------------------------
.print:
	cmp rcx, 0
	je .empty

	call print_dirdp

	jmp .end

;------- end DIR --------------------------------


	; MAKE
;---------------------------------------
.make:
	cmp rcx, MAX_NUM_RECS		; Max number of dirs or files
	je .full

	cmp rcx, 0
	je .new
	call search_dirf		; check if file/dir is already present
	jc .already

.new:
	inc rcx				; skip header and
	mov qword[rsi], rcx		; add new item in counter
.again:
	add rsi, RECORD_SIZE
	dec rcx
	cmp rcx, 0
	jne .again

	mov qword[new_item], rsi	; now rsi points to the room for new item

 ; Fill 128 bytes with 0 
 ;-----------------------
	mov rdi, [new_item]
	mov rcx, RECORD_SIZE  
.fill:
	mov byte [rdi], 0
	inc rdi
	dec rcx				     
	cmp rcx, 0
	jne .fill
	;---------------

	mov rdi, [new_item]

	call get_cmos_data
	call set_string_datetime
	mov rsi, string_datetime
	call os_string_to_int
	mov qword[rdi + 80], rax	; datetime

	mov rax, [df_size]
	mov qword[rdi + 72], rax	; size (empty)

	mov rax, [level]
	add al, 2			; new name (level+1=>strv+2)
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; in rsi
.mk2:
	cmp [c_idx], 2			; MKD		[c_idx] = 2
	jne .mk3

	call set_dirf_name
	call os_string_uppercase	; always uppercase	
	call os_string_copy		; directory name
	jmp .newdirectory

.mk3:
					; MKF	 	[c_idx] = 3 or 7
	call set_dirf_name
	call os_string_lowercase	; always lowercase	
	call os_string_copy		; file name
	jmp .newfile


 ;----------------------------------
 ;  The new directory (level+1)
 ;----------------------------------
.newdirectory:

	mov rdi, cluster_buffer2	; Fill the buffer...
	mov rcx, CLUSTER_SIZE  
.fill2:
	mov byte [rdi], 0		; ...of new dir with 0
	inc rdi
	dec rcx				     
	cmp rcx, 0
	jne .fill2
	;------------------

	mov rdi, cluster_buffer2
	mov qword[rdi + 8], RECORD_SIZE		; fixed lenght
	mov rax, [level]
	inc rax
	mov qword[rdi + 32], rax		; level+1

	mov rax, 'END0-DIR'
	mov qword[rdi + 120], rax		; end rec.0
	
	mov rbx, cluster_buffer2
	mov rcx, [df_size]		; 508*4096
	mov rdx, 1			; new dir
	call fs_save_file
	jc .error

	jmp .save_ptrs
 ;----------------------------

 ;----------------------------------
 ;  The new file
 ;----------------------------------
.newfile:	
	mov rbx, [pm_file]		; EOF / ptr to file in memory
	mov rcx, [df_size]		; 1   / size of file
	mov rdx, 0			; new file / STO
	call fs_save_file
	jc .error

	;jmp .save_ptrs
 ;----------------------------


.save_ptrs:
	mov rdi, [new_item]
	mov qword[rdi + 88], rcx	; pointer to first cluster of pointers
	mov qword[rdi + 96], rdx	; pointer to last cluster of pointers


	;---------------------
	cmp [level], 3
	jne .save_dl

	mov rsi, r9			; directory of level
	mov rcx, qword[rsi + 16]	; total size of files (in level 3)
	mov rax, [df_size]		; 1 / size
	add rcx, rax
	mov qword[rsi + 16], rcx
	;---------------------

.save_dl:
	mov rax, r10			; cluster of pointers, lba (directory l1,l2 or l3)
	mov rbx, r9			; ptr to file in memory
	mov rsi, r9			; directory of level
	mov rcx, qword[rsi]		; num. dirs or files
	inc rcx				; num. records
	shl rcx, 7			; mul 128 = size

	call fs_save_directory
	jc .error

	;--------------
	cmp [c_idx], 7			; STO
	je .end_mkf
	;--------------

	jmp .OK

;---- end MAKE -----------------------------------


	; REN
;---------------------------------------
.replace:
	cmp rcx, 0
	je .not_dir_file

	mov rdi, rsi

	mov rax, [level]
	add al, 2
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name (old)

.againS:				; search
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found
	dec rcx
	cmp rcx, 0
	jne .againS
	jmp .not_dir_file

.found:
	mov rax, [level]
	add al, 3
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name (new)

	cmp [level], 3
	je .lowcase
	jmp .upcase

.lowcase:
	call set_dirf_name
	call os_string_lowercase
	jmp .copy
.upcase:
	call set_dirf_name
	call os_string_uppercase

.copy:
	call os_string_copy

	;------------

	mov rax, r10			; cluster of pointers, lba (directory l1,l2 or l3)
	mov rbx, r9			; ptr to file in memory
	mov rsi, r9			; directory of level
	mov rcx, qword[rsi]		; num. dirs or files
	inc rcx				; num. records
	shl rcx, 7			; mul 128 = size

	call fs_save_directory
	jc .error

	jmp .OK

;---- end REN -----------------------------------

	; DEL
;---------------------------------------
.delete:
	cmp rcx, 0
	je .not_dir_file

	mov rdi, rsi

	mov rax, [level]
	add al, 2
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name

.againSn:				; search
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found_n
	dec rcx
	cmp rcx, 0
	jne .againSn
	jmp .not_dir_file

.found_n:
	;-----------------------
	mov rax, qword[rdi + 72]	; size
	mov [df_size], rax		; store value
	mov rdx, 0			; delete file

	cmp [level], 3			; if file then delete
	je .del_f

	mov [df_size], 508*4096		; size of directory
	mov rdx, 1			; delete dir

	; Delete file
	;--------------------------------------
.del_f:
	mov rax, qword[rdi + 88]	; pointer to first cluster of pointers
	mov rbx, qword[rdi + 96]	; pointer to last cluster of pointers
	mov rcx, [df_size]
					; rdx = 0 or 1
	call fs_erase_file		; Free the file
	jc .error
	;----------------------

	; Copy last record on record to delete
	;---------------------------------------
	mov rsi, r9			; pointer to memory (directory)
	mov rcx, qword[rsi]		; num. dirs or files
	shl rcx, 7			; mul 128 = size
	add rsi, rcx			; now point to last record

	; Copy 128 bytes
	mov rcx, RECORD_SIZE  
.crec:
	mov dl, byte [rsi]
	mov byte [rdi], dl
	mov byte [rsi], 0		; delete byte of last record
	inc rdi
	inc rsi
	dec rcx				     
	cmp rcx, 0
	jne .crec
	;---------------

	; Update number and size of files
	;---------------------------------------
	mov rsi, r9			; pointer to memory (directory)
	mov rcx, qword[rsi]		; num. dirs or files
	dec rcx
	mov qword[rsi], rcx

	cmp [level], 3
	jne .save_dl2

	mov rcx, qword[rsi + 16]	; total size of files (in level 3)
	mov rax, [df_size]
	sub rcx, rax
	mov qword[rsi + 16], rcx

.save_dl2:
	mov rax, r10			; cluster of pointers, lba (directory l1,l2 or l3)
	mov rbx, r9			; ptr to file in memory
	mov rsi, r9			; directory of level
	mov rcx, qword[rsi]		; num. dirs or files
	inc rcx				; num. records
	shl rcx, 7			; mul 128 = size

	call fs_save_directory
	jc .error

	;--------------
	cmp [c_idx], 7			; STO
	je .end_del
	;--------------

	jmp .OK

;---- end DEL -----------------------------------


	; LIM
;---------------------------------------
.load_in_memory:
	cmp rcx, 0
	je .not_dir_file

	mov rdi, rsi

	mov rax, [level]
	add al, 2
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name

.againSl:				; search
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found_l
	dec rcx
	cmp rcx, 0
	jne .againSl
	jmp .not_dir_file
.found_l:

	;-----------------------
	mov rcx, qword[rdi + 72]	; size
	shr rcx, 21			; div 2 MiB
	add rcx, 1			; num. pages to allocate
	call os_mem_allocate
	cmp rcx, 0
	je .error
	;-----------------------

	mov rdx, rax			; store value
	mov rbx, rax			; ptr to file in memory
	mov rax, qword[rdi + 88]	; pointer to first cluster of pointers

	call fs_load_file
	jc .errorM			; release memory

	; OUT
	mov rax, rdx			; ptr to file in memory
	mov rbx, qword[rdi + 72]	; size

	clc
	jmp .end

.errorM:
	mov rax, rdx
					; rcx = num. pages
	call os_mem_release
	
	jmp .error


;---- end LIM -----------------------------------


	; STO
;---------------------------------------
.store:
	jmp .delete			; Delete the old file
.end_del:

	; re-init values
	mov rax, [sto_file]
	mov [pm_file], rax
	mov rax, [sto_size]
	mov [df_size], rax

	mov rsi, r9			; pointer to memory (directory)
	mov rcx, qword[rsi]		; num. dirs or files

	jmp .make			; Save the new file
.end_mkf:

	clc
	jmp .end


;---- end STO -----------------------------------


	; RUN
;---------------------------------------
.run:
	cmp rcx, 0
	je .not_dir_file

	mov rdi, rsi

	mov rax, [level]
	add al, 2
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name

.againSr:				; search
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found_r
	dec rcx
	cmp rcx, 0
	jne .againSr
	jmp .not_dir_file
.found_r:

	;----------------------- memory map
	call K_mmap_prg_release		; release memory of previous program
	mov rcx, qword[rdi + 72]	; size
	call K_mmap_prg_add		; mark memory
	;-----------------------

	mov rbx, programlocation	; ptr to program in memory
	mov rax, qword[rdi + 88]	; pointer to first cluster of pointers

	call fs_load_file
	jc .error

	clc
	jmp .end


;---- end RUN -----------------------------------


.already:
	mov rsi, msg_already
	jmp .print_err
.not_dir_file:
	mov rsi, msg_no_dir_file
	jmp .print_err
.full:
	mov rsi, msg_full
	jmp .print_err
.empty:
	mov rsi, msg_empty
	jmp .print_err
.not_empty:
	mov rsi, msg_not_empty
	jmp .print_err
.root:
	mov rsi, msg_no_change
	jmp .print_err
.no_arg:
	mov rsi, msg_no_args
	jmp .print_err
.nofs:
	mov rsi, msg_no_fs
	jmp .print_err
.error:
	mov rsi, msg_error
	jmp .print_err

.OK:
	mov rsi, msg_end_command
	call os_print_string
	call os_print_newline
	clc
	jmp .end

.print_err:
	call os_print_string
	call os_print_newline
	stc
	;jmp .end

.end:
	pop r11
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx

ret


;---------------------------------------------------------------------------
; Copy - Move File commands
; IN : 	rsi = command arguments string (breaked)
; OUT:	none regs
;	set carry
fs_command2:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8
	push r9
	push r10
	push r11

	mov r9, rsi			; store value (brk string pointer)
	mov rdx, rsi			; store value (brk string pointer)
	;----

	mov r8, qword [rsi]		; Save 8 chars

	mov rsi, lim_string		; LIM,0
	mov rdi, r9
	call os_string_copy
	mov rsi, r9

	mov rcx, 6
	call fs_command
	jc .err_command

	mov r10, rax			; store value (ptr file LIM)
	mov r11, rbx			; store value (size LIM)

	mov rsi, r9			; Point to first word
	mov qword [rsi], r8		; Restore 8 chars
	;----

	mov al, 6
	mov ah, 0
	mov rsi, r9
	call os_get_strv		; rsi = start of destination device string

	sub rsi, 4			; room for command
	mov r9, rsi			; store new value (destination)
	mov r8, qword [rsi]		; Save 8 chars

	mov rsi, mkf_string		; MKF,0
	mov rdi, r9
	call os_string_copy
	mov rsi, r9			; Point to first word (destination)

	mov rcx, 6
	call fs_command
	jc .err_command
	;----

	mov rax, r10			; restore value (ptr file from LIM)
	mov rbx, r11			; restore value (size from LIM)

	mov rsi, sto_string		; STO,0
	mov rdi, r9
	call os_string_copy
	mov rsi, r9			; Point to first word (destination)

	mov rcx, 6
	call fs_command
	jc .err_command

	mov rsi, r9			; Point to first word (destination)
	mov qword [rsi], r8		; Restore 8 chars
	;----

	mov r9, rdx			; Point to first word
	mov rsi, r9
	mov di, word[rsi]		; First 2 chars
	cmp di, 'CP'
	je .OK
	;----
	
	mov r8, qword [rsi]		; Save 8 chars

	mov rsi, del_string		; DEL,0
	mov rdi, r9
	call os_string_copy
	mov rsi, r9

	mov rcx, 6
	call fs_command
	jc .err_command

	mov rsi, r9			; Point to first word
	mov qword [rsi], r8		; Restore 8 chars
	;----

	;----
	mov ah, [screen_cursor_x]	; remove the double 'command completed'
	mov al, [screen_cursor_y]
	dec al
	call os_row_clear
	;----

.OK:
	clc
	jmp .end

.err_command:
	stc
.end:
	pop r11
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret


;--------------------------------
; IN :	rsi = name
; OUT:	none regs
set_dirf_name:
	push rax
	push rbx
	push rsi

	mov bl, '_'
	mov al, 32			; space
	call os_string_change_char
	mov al, 34			; "
	call os_string_change_char
	mov al, 39			; '
	call os_string_change_char
	mov al, 42			; *
	call os_string_change_char
	mov al, 47			; /
	call os_string_change_char
	mov al, 92			; \
	call os_string_change_char
	mov al, 96			; `
	call os_string_change_char
	mov al, 124			; |
	call os_string_change_char
	mov al, 126			; ~
	call os_string_change_char

	pop rsi
	pop rbx
	pop rax

	ret

;--------------------------------
; IN :	rsi = pointer to memory
;	rcx = num. of dirs or files
; OUT:	none regs
;	set carry
search_dirf:
	push rax
	push rcx
	push rsi
	push rdi

	mov rdi, rsi

	mov rax, [level]
	add al, 2
	mov ah, 0
	mov rsi, [c_string]
	call os_get_strv		; rsi = name

.again:	
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found
	dec rcx
	cmp rcx, 0
	jne .again
	jmp .not_found

.found:
	stc
	jmp .end
.not_found:
	clc
.end:
	pop rdi
	pop rsi
	pop rcx
	pop rax
	ret


;--------------------------------
; IN :	rsi = pointer to memory
;	rcx = num. of directories
print_dirdp:
	push rax
	push rbx
	push rdx
	push rcx
	push r8
	push r9
	push r10

	mov r8, qword[rsi+16] 	; total size
	mov r9, qword[rsi+32] 	; level
	mov r10, rcx		; n.files
;-------
	xor rdx, rdx
	call os_print_newline
.again:
	cmp rdx, 20
	jne .next
	xor rdx, rdx
	call os_print_newline
	push rsi
	mov rsi, msg_spacebar
	call os_print_string
	call os_print_newline
	call os_print_newline
	pop rsi
	call os_input_key_wait
	cmp al, ' '
	je .next
	jmp .end

.next:
	add rsi, RECORD_SIZE
	call os_print_string
;---
	push rsi

	add rsi, 72
	mov rax, qword[rsi]	; <---
	add rsi, 8
	mov rbx, qword[rsi]	; <---

	mov rsi, space
	call os_print_string

	cmp [screen_cursor_x], 57
	ja .skip		; skip the rest of line

	mov rdi, temp_str
	call os_int_to_string	; rax->string = size
	mov rsi, temp_str
	call os_print_string

	mov rsi, space
	call os_print_string

	cmp [screen_cursor_x], 65
	ja .skip		; skip the rest of line

	mov rdi, string_datetime	
	mov rax, rbx		; rbx->string = datetime
	call os_int_to_string
	mov rsi, string_datetime
	call os_print_string

.skip:
	pop rsi
;---
	call os_print_newline
	inc rdx
	dec rcx
	cmp rcx, 0
	jne .again

;-------
	cmp r9, 3
	jne .dironly

	call os_print_newline
	mov rax, r8
	mov rdi, temp_str
	call os_int_to_string	; rax->string = total size
	mov rsi, temp_str
	call os_print_string
	mov rsi, msg_bytes
	call os_print_string
	mov rax, r10
	mov rdi, temp_str
	call os_int_to_string	; rax->string = n.files
	mov rsi, temp_str
	call os_print_string
	mov rsi, msg_bytes2
	call os_print_string

.dironly:
	call os_print_newline
.end:
	pop r10
	pop r9
	pop r8
	pop rdx
	pop rcx
	pop rbx
	pop rax
ret


;----------------------------
; fs_load_header
; IN :	eax = device
; OUT:	none regs
;	set carry
fs_load_header:				; used by fs_probe
	push rax
	push rcx
	push rdi

	call current_device
	jc .error
	cmp [current.fs], 0
	je .error

	mov rax, LBA_HEADER1
	mov rdi, [current.header]
	call read_cluster
	cmp rcx, 0
	je .error

	jmp .ok

.error:
	stc
	jmp .end
.ok:
	clc
.end:
	pop rdi
	pop rcx
	pop rax

	ret

;----------------------------
; fs_load_level0
; IN :	eax = device
; OUT:	none regs
;	set carry
fs_load_level0:				; used by fs_probe
	push rax
	push rcx
	push rdi

	call current_device
	jc .error
	cmp [current.fs], 0
	je .error

	mov rax, LBA_ROOT		; directly (no pointers)
	mov rdi, [current.root]
	call read_cluster
	cmp rcx, 0
	je .error

	jmp .ok

.error:
	stc
	jmp .end
.ok:
	clc
.end:
	pop rdi
	pop rcx
	pop rax

	ret


;--------------------------------------------
; fs_load_level_n
; IN :	rsi = pointer to memory (n-1)
;	rcx = num. of directories (n-1)
;	rax = level (n)
; OUT:	rsi = pointer to memory (n)
;	rcx = num. of directories (n)
;	rax = (first) cluster of pointers (n), lba
;	rbx = (last) cluster of pointers (n), lba
;	set carry
fs_load_level_n:	
	push rdx
	push rdi
	push r8
	push r9
	push r10

	cmp rax, 0
	je .l0	

	mov rdi, rsi

	mov rsi, [c_string]		; strv -> directory level
	inc rax				; ah = 0			
	call os_get_strv		; rsi = name
;-------
	cmp rcx, 0
	je .error
;-------
.again:					; search
	add rdi, RECORD_SIZE
	call os_string_compare
	jc .found
	dec rcx
	cmp rcx, 0
	jne .again
	jmp .error
.found:

	;------------------------------- Caching (Search)
	mov rsi, directory_cache_ptrs
	xor rax, rax
	mov eax, dword[current.string]	; device
	mov rbx, qword[rdi+88]		; (first) cluster of pointers, lba
	mov rcx, 12
.next:
	cmp rax, qword[rsi+0]		; device
	jne .ndc	
	cmp rbx, qword[rsi+8]		; lba
	jne .ndc
	mov r10, qword[rsi+16]		; memory
	mov r9, qword[rdi+96]		; (last)
	mov r8, qword[rdi+88]		; (first)
	jmp .ok
.ndc:
	add rsi, 24			; next (item's size)
	dec rcx
	cmp rcx, 0
	jne .next
	;------------------------------- (End)

	mov rcx, 1			; num. pages to allocate
	call os_mem_allocate
	cmp rcx, 0
	je .error

	mov r10, rax			; store the value
	mov rbx, rax			; pointer to file in memory 
	mov rax, qword[rdi+96]		; (last) cluster of pointers, lba
	mov r9, rax			; store the value
	mov rax, qword[rdi+88]		; (first) cluster of pointers, lba
	mov r8, rax			; store the value
	call fs_load_directory
	jc .error

	;------------------------------- Caching (Add)
	mov rsi, directory_cache_ptrs
	add rsi, 264+16			; now point to last memory ptr
	mov rax, qword[rsi]
	mov rcx, 1
	call os_mem_release		; release page memory (no check error)

	mov rsi, directory_cache_ptrs
	mov rcx, 11
	add rsi, 240			; now point to item n.11
	
.nac:					; move 11 times
	mov rax, qword[rsi+0]
	mov qword[rsi+0+24], rax
	mov rax, qword[rsi+8]
	mov qword[rsi+8+24], rax
	mov rax, qword[rsi+16]
	mov qword[rsi+16+24], rax

	sub rsi, 24
	dec rcx
	cmp rcx, 0
	jne .nac

	mov rsi, directory_cache_ptrs	; add new item
	xor rax, rax
	mov eax, dword[current.string]		
	mov qword[rsi+0], rax		; device
	mov qword[rsi+8], r8		; (first)
	mov qword[rsi+16], r10		; ptr to memory
	;------------------------------- (End)

	jmp .ok

.error:
	stc
	jmp .end
.l0:
	mov rsi, [current.root]
	mov rcx, qword[rsi]		; num. directories (8)
	mov rax, LBA_ROOT_P		; don't use it!
	mov rbx, LBA_ROOT_P		; don't use it!
	clc
	jmp .end
.ok:
	mov rsi, r10			; ptr to memory
	mov rcx, qword[rsi]		; num. directories
	mov rax, r8			; (first) cluster of pointers, lba
	mov rbx, r9			; (last) cluster of pointers, lba
	clc
.end:
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rdx
	ret

;---------------------------------------------------------;
; fs_load_directory
; IN:	rax = first cluster of pointers
;	rbx = ptr to file in memory
; OUT:	set carry
fs_load_directory:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8

	mov [file_ptr], rbx
	mov [file_ptr_off], 0
	mov [ncluster_d], 0

.read:
	mov rdi, cluster_buffer
	call read_cluster
	cmp rcx, 0
	je .error

	mov rsi, cluster_buffer
	add rsi, CC_PTRS_SIZE
.next:
	mov rax, qword[rsi]

	cmp rax, 0
	je .end_pointers

	mov rdi, [file_ptr]
	add rdi, [file_ptr_off]

	call read_cluster
	cmp rcx, 0
	je .error

	cmp [ncluster_d], 0
	jne .check
	;-----				; calculate ncluster_d
	mov rbx, [file_ptr]
	add rbx, [file_ptr_off]
	mov rcx, qword[rbx]		; num. dirs or files
	inc rcx				; num. records
	shl rcx, 7			; mul 128 = size

	xor  rdx, rdx
	mov  rax, rcx			; rdx:rax
	mov  rcx, CLUSTER_SIZE
	div  rcx
	cmp  rdx, 0			; rdx = remainder
	je   .s1
	inc  rax			; rax = quotient
.s1:
	mov [ncluster_d], rax		; save value
	;-----

.check:
	dec [ncluster_d]	
	cmp [ncluster_d], 0		; only data
	je .end_pointers

	add [file_ptr_off], CLUSTER_SIZE
	add rsi, 8
	jmp .next

.end_pointers:
	; only one cluster of pointers
	;mov rsi, cluster_buffer
	;mov rax, qword[rsi]
	;cmp rax, 0
	;je .last
	;jmp .read

.last:
	clc
	jmp .end
.error:
	stc
.end:
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret



;----------------------------------------------
; fs_load_file					
; IN:	rax = first cluster of pointers
;	rbx = ptr to file in memory
; OUT:	set carry
fs_load_file:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov [file_ptr], rbx
	mov [file_ptr_off], 0

.read:
	mov rdi, cluster_buffer
	call read_cluster
	cmp rcx, 0
	je .error

	mov rsi, cluster_buffer
	add rsi, CC_PTRS_SIZE
.next:
	mov rax, qword[rsi]

	cmp rax, 0
	je .end_pointers

	mov rdi, [file_ptr]
	add rdi, [file_ptr_off]

	call read_cluster
	cmp rcx, 0
	je .error

	add [file_ptr_off], CLUSTER_SIZE
	add rsi, 8
	jmp .next
.end_pointers:

	mov rsi, cluster_buffer
	mov rax, qword[rsi]
	cmp rax, 0
	je .last
	jmp .read			; next cluster of pointers

.last:
	clc
	jmp .end
.error:
	stc
.end:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret

;-----------------------


;----------------------------------------------
; fs_erase_file
; IN:	rax = first cluster of pointers
;	rbx = last cluster of pointers
;	rcx = size
;	rdx = 0 if file, 1 if dir
; OUT:	set carry
fs_erase_file:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8
	push r9
	push r10
	push r11

	mov r10, rax				; the first cl. of pointers to file (lba)
	mov r11, rbx				; the last cl. of pointers to file (lba)
	mov [file_size], rcx
	mov [file_type], rdx


.start:
	mov rax, r10
	mov rsi, [current.header] 
	mov r8, qword[rsi + 48]			; First free cluster chain (lba)
	mov r9, qword[rsi + 56]			; Last free cluster chain (lba)

	; read (and write) first cluster of pointers to file
	mov rdi, cluster_buffer
	call read_cluster
	cmp rcx, 0
	je .error

	mov rax, r10
	mov rsi, cluster_buffer
	mov qword[rsi + 8], r9			; prev (file) <- lba last (chain)

	call write_cluster
	cmp rcx, 0
	je .error

	;----------------------

	; read (and write) last cluster of chain
	mov rax, r9
	mov rdi, cluster_buffer
	call read_cluster
	cmp rcx, 0
	je .error

	mov rax, r9
	mov rsi, cluster_buffer
	mov qword[rsi], r10			; next (chain) <- lba first (file)

	call write_cluster
	cmp rcx, 0
	je .error
	;----------------------

	; update (and write) last lba in header
	mov rsi, [current.header] 
	mov qword[rsi + 56], r11		; lba last (chain) <- lba last (file)


 ; Number of clusters
 ;----------------------------------------------------;
	xor  rdx, rdx
	mov  rax, [file_size]	; rdx:rax
	mov  rcx, CLUSTER_SIZE
	div  rcx
	cmp  rdx, 0		; rdx = remainder
	je   .s1
	inc  rax		; rax = quotient
.s1:
	mov [bytes_last], rdx	; save value
	xor  rdx, rdx
	mov [ncluster_d], rax	; save value
	mov rcx, NUM_POINTERS
	div rcx
	cmp  rdx, 0		; rdx = remainder
	je   .s2
	inc  rax		; rax = quotient
.s2:
	mov [ncluster_p], rax	; save value

 ; Write headers
 ;----------------------------------------------------;
.final:
	mov rsi, [current.header]
	mov rax, [rsi + 16]			; Free Space Left (in cluster)
	add rax, [ncluster_p]
	add rax, [ncluster_d]
	mov qword [rsi + 16], rax		; write in memory

	cmp [file_type], 0			; if file
	jne .dtot
	mov rax, [rsi + 24]			; Total number of files
	dec rax
	mov qword [rsi + 24], rax		; write in memory
	jmp .writeheader
.dtot:
	cmp [file_type], 1			; if directory
	jne .writeheader
	mov rax, [rsi + 32]			; Total number of directories
	dec rax
	mov qword [rsi + 32], rax		; write in memory

.writeheader:
	mov rsi, [current.header]
	mov rax, LBA_HEADER1
	call write_cluster			; write header1 on device
	mov rsi, [current.header]
	mov rax, LBA_HEADER2
	call write_cluster			; write header2 on device
						; no check error

; Update cache
;-------------------------
	cmp [file_type], 0			; if file
	je .OK

	;------------------------------- Caching (Search)
	mov rsi, directory_cache_ptrs
	xor rax, rax
	mov eax, dword[current.string]	; device
	mov rbx, r10			; (first) cluster of pointers, lba
	mov rcx, 12
.next:
	cmp rax, qword[rsi+0]		; device
	jne .ndc	
	cmp rbx, qword[rsi+8]		; lba
	jne .ndc
	; clear the item
	mov qword[rsi+16], 0		; memory
	mov qword[rsi+8], 0		; lba
	mov qword[rsi+0], 0		; device
	jmp .OK
.ndc:
	add rsi, 24			; next (item's size)
	dec rcx
	cmp rcx, 0
	jne .next
	;------------------------------- (End)


.OK:
	clc
	jmp .end
.error:
	stc
.end:
	pop r11
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret

;-----------------------


;---------------------------------------------------------;
; fs_save_directory
; IN:	rax = first cluster of pointers
;	rbx = ptr to file in memory
;	rcx = size (only data)
; OUT:	set carry
fs_save_directory:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r9

	mov [lba_directory], rax
	mov [file_ptr], rbx
	mov [file_ptr_off], 0
	mov [file_size], rcx

	xor  rdx, rdx
	mov  rax, [file_size]	; rdx:rax
	mov  rcx, CLUSTER_SIZE
	div  rcx
	cmp  rdx, 0		; rdx = remainder
	je   .s1
	inc  rax		; rax = quotient
.s1:
	mov [ncluster_d], rax	; save value

 ; Read pointers cluster
 ;----------------------------------------------------;
	mov rax, [lba_directory]
	mov rdi, cluster_buffer
	call read_cluster
	cmp rcx, 0
	je .error

 ; Write data clusters
 ;----------------------------------------------------;
.write:
	mov r9, [ncluster_d]
	mov rdi, cluster_buffer
	add rdi, CC_PTRS_SIZE
.next_d:
	mov rax, qword[rdi]			; lba
	mov rsi, [file_ptr]
	add rsi, [file_ptr_off]

	call write_cluster			; data cluster
	cmp rcx, 0
	je .error

	add rdi, POINTER_SIZE
	add [file_ptr_off], CLUSTER_SIZE
	dec r9
	cmp r9, 0
	jne .next_d

.OK:
	clc
	jmp .end
.error:
	stc
.end:
	pop r9
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret



;---------------------------------------------------------;
; fs_save_file
; IN:	rbx = ptr to file in memory
;	rcx = size
;	rdx = 0 if new file, 1 if new dir
; OUT:	rcx = first cluster of pointers, lba
;	rdx = last cluster of pointers, lba
;	set carry
;---------------------------------------------------------;

fs_save_file:

	push rax
	push rbx
	push rsi
	push rdi
	push r9
	push r10

	mov [file_ptr], rbx
	mov [file_ptr_off], 0
	mov [file_size], rcx
	mov [file_type], rdx

 ; Read header
 ;----------------------------------------------------;
	xor  rdx, rdx
	mov  rax, [file_size]	; rdx:rax
	mov  rcx, CLUSTER_SIZE
	div  rcx
	cmp  rdx, 0		; rdx = remainder
	je   .s1
	inc  rax		; rax = quotient
.s1:
	mov [bytes_last], rdx	; save value
	xor  rdx, rdx
	mov [ncluster_d], rax	; save value
	mov rcx, NUM_POINTERS
	div rcx
	cmp  rdx, 0		; rdx = remainder
	je   .s2
	inc  rax		; rax = quotient
.s2:
	mov [ncluster_p], rax	; save value

.s3:
	mov rsi, [current.header]
	mov rax, [rsi + 16]	; Free space left (in cluster)
	mov [freespace], rax

	mov rax, [ncluster_p]
	add rax, [ncluster_d]
	cmp [freespace], rax
	jb .error		; no space

	;-----------------------


 ; Allocate memory
 ;----------------------------------------------------;
	mov rcx, [ncluster_p]
	add rcx, [ncluster_d]
	inc rcx			; null terminated d
	inc rcx			; null terminated p
	shl rcx, 3		; mul 8 = bytes to allocate for pointers
	shr rcx, 21		; div 2 MiB
	inc rcx			; num. of pages
	mov [npages], rcx
	call os_mem_allocate
	cmp rcx, 0
	je .error
	mov [mem_pages], rax
	mov [mem_pages_off], 0

	;-----------------------


 ; Read first cluster with pointers
 ;----------------------------------------------------;
	mov rsi, [current.header]
	mov rax, [rsi + 48]			; First free cluster chain absolute address
	mov rdi, cluster_buffer
	call read_cluster			; Read
	cmp rcx, 0
	je .errorM


 ; Copy pointers in memory
 ;----------------------------------------------;
	mov r10, [ncluster_d]
	add r10, [ncluster_p]			; total num. of pointers

.read_pointers1:
	mov rsi, [current.header]
	mov rax, [rsi + 64]			; first use
	cmp rax, 0
	je .read_pointers2			;

	mov rax, [rsi + 72]			; first free cluster
	mov rdx, rax				; save value
	add rax , SECTORS_CLUSTER		; +8, the next cluster is the new first free
	mov qword [rsi + 72], rax		; write in memory

	mov rax, [rsi + 64]			; first use
	dec rax					; sub the cluster to use from counter
	mov qword [rsi + 64], rax		; write in memory
	jmp .copy_pointers

;----
.read_pointers2:
	mov rdi, cluster_buffer
	mov rbx, qword [rdi + 4088]		; offset
	mov rdx, qword [rdi + rbx]		; first free cluster
	cmp rdx, 0
	je .nextcluster				; empty
	mov qword [rdi + rbx], 0		; clear the pointer
	add rbx, POINTER_SIZE
	mov qword [rdi + 4088], rbx		; new offset
	jmp .copy_pointers

.nextcluster:
	mov rsi, [current.header]
	mov rdx, [rsi + 48]			; First free cluster chain absolute address (itself, empty)

	mov rdi, cluster_buffer
	mov rax, qword [rdi]			; Next pointer (First free cluster chain absolute address)

	mov rsi, [current.header]
	mov qword[rsi + 48], rax		; new First free cluster chain absolute address

	call read_cluster			; re-read cluster with pointers to free clusters
	cmp rcx, 0
	je .errorM

	mov rdi, cluster_buffer
	mov qword [rdi + POINTER_SIZE], 0	; clear prev pointer
	jmp .copy_pointers
;----

.copy_pointers:
	mov rbx, [mem_pages]
	add rbx, [mem_pages_off]
	mov qword[rbx], rdx			; copy pointer in memory
	add [mem_pages_off], POINTER_SIZE	; next pointer

	dec r10
	
	cmp r10, [ncluster_p]
	jne .check0

	mov rbx, [mem_pages]
	add rbx, [mem_pages_off]
	mov qword[rbx], 0			; null terminated d
	add [mem_pages_off], POINTER_SIZE	; next pointer

.check0:
	cmp r10, 0
	jne .read_pointers1
	

	mov rbx, [mem_pages]
	add rbx, [mem_pages_off]
	mov qword[rbx], 0			; null terminated p

	jmp .write
	;-----------------------

;-------- end read pointers -----------------------------;



 ; Write cluster with pointers to free clusters
 ;----------------------------------------------------;
.write:
	mov rsi, [current.header]
	mov rax, [rsi + 48]			; First free cluster chain absolute address
	mov rsi, cluster_buffer
	call write_cluster			; Write
	cmp rcx, 0
	je .errorM


 ; Write data clusters
 ;----------------------------------------------------;
	mov r9, [ncluster_d]
	mov [mem_pages_off], 0
.next_d:
	mov rbx, [mem_pages]
	add rbx, [mem_pages_off]
	mov rax, qword[rbx]			; lba

	mov rsi, [file_ptr]
	add rsi, [file_ptr_off]

	;----
	cmp [file_type], 1			; if new directory and...
	jne .slc				;
	cmp r9, 508				; ...not the first cluster
	jne .addptr				; - not write -
	;----
.slc:
	cmp r9, 1				; last cluster
	jne .wdc
	cmp [bytes_last], 0			; check if full
	je .wdc	
	call set_last_cluster			; clear buffer and copy data in (only data bytes)-> rsi
.wdc:
	call write_cluster			; data cluster
	cmp rcx, 0
	je .errorM

.addptr:
	add [mem_pages_off], POINTER_SIZE
	add [file_ptr_off], CLUSTER_SIZE
	dec r9

	cmp r9, 0
	jne .next_d

	mov rax, [mem_pages]
	add rax, [mem_pages_off]		; now point to zero delimeter
	add rax, POINTER_SIZE			; now point to first pointer of clusters of pointers
	mov [mem_pages_p], rax
	mov rbx, qword[rax]
	mov [first_lba_file], rbx		; the first pointer to file


 ; Fill the buffer and write the clusters of pointers
 ;----------------------------------------------------;
	mov r9, [ncluster_p]
	mov [mem_pages_off], 0
.next_p:
	mov rcx, NUM_POINTERS 
	mov rdi, cluster_buffer
	mov qword [rdi + 4080], 0
	mov qword [rdi + 4088], CC_PTRS_SIZE	; 16
	add rdi, CC_PTRS_SIZE
.fill1:
	mov rax, [mem_pages]
	add rax, [mem_pages_off]

	mov rbx, qword [rax]
	mov qword [rdi], rbx

	cmp qword [rax], 0			; until null terminated...
	je .end_fill1

	add rdi, POINTER_SIZE
	add [mem_pages_off], POINTER_SIZE
	dec rcx				     
	cmp rcx, 0				; ...or 508 copied
	jne .fill1

.end_fill1:
	mov rdi, cluster_buffer

	mov rbx, qword [mem_pages_p]
	add rbx, POINTER_SIZE			; next
	mov rax, qword[rbx]			; lba
	mov qword [rdi + 0], rax

	mov rbx, qword [mem_pages_p] 
	sub rbx, POINTER_SIZE			; prev
	mov rax, qword[rbx]			; lba
	mov qword [rdi + 8], rax

	mov rbx, [mem_pages_p]
	mov rax, qword[rbx]			; lba
	mov rsi, cluster_buffer

	call write_cluster			; pointers cluster
	cmp rcx, 0
	je .errorM

	add [mem_pages_p], POINTER_SIZE

	dec r9
	cmp r9, 0
	jne .next_p

	mov rbx, qword [mem_pages_p]
	sub rbx, POINTER_SIZE
	mov rax, qword[rbx]			; lba
	mov [last_lba_file], rax		; the last pointer to file

	;-----------------------
	;jmp .final



.final:
	; write header
	;------------------------
	mov rsi, [current.header]
	mov rax, [rsi + 16]			; Free Space Left (in cluster)
	sub rax, [ncluster_p]
	sub rax, [ncluster_d]
	mov qword [rsi + 16], rax		; write in memory

	cmp [file_type], 0			; if new file
	jne .dtot
	mov rax, [rsi + 24]			; Total number of files
	inc rax
	mov qword [rsi + 24], rax		; write in memory
	jmp .writeheader
.dtot:
	cmp [file_type], 1			; if new directory
	jne .writeheader
	mov rax, [rsi + 32]			; Total number of directories
	inc rax
	mov qword [rsi + 32], rax		; write in memory

.writeheader:
	mov rax, LBA_HEADER1
	call write_cluster			; write header1 on device
	mov rsi, [current.header]
	mov rax, LBA_HEADER2
	call write_cluster			; write header2 on device
						; no check error

	mov rax, [mem_pages]
	mov rcx, [npages]
	call os_mem_release	; no check error

	clc
	jmp .end

.errorM:
	mov rax, [mem_pages]
	mov rcx, [npages]
	call os_mem_release	; no check error
.error:	
	stc
.end:
	pop r10
	pop r9
	pop rdi
	pop rsi
	pop rbx
	pop rax

	mov rcx, [first_lba_file]
	mov rdx, [last_lba_file]
	ret

;------------------------------------
; set_last_cluster
; IN : rsi = ptr mem. of data
; OUT: rsi = cluster buffer zeroed + data only
set_last_cluster:

	push rax
	push rcx
	push rdi

	; Fill the buffer of cluster with 0
	mov rcx, CLUSTER_SIZE
	mov rdi, cluster_buffer
.fill0:
	mov byte [rdi], 0
	inc rdi
	dec rcx				     
	cmp rcx, 0
	jne .fill0

	; Fill the buffer of cluster with data
	mov rcx, [bytes_last]
	mov rdi, cluster_buffer
.fill1:
	mov al, byte [rsi]
	mov byte [rdi], al
	inc rdi
	inc rsi
	dec rcx				     
	cmp rcx, 0
	jne .fill1

	mov rsi, cluster_buffer

	pop rdi
	pop rcx
	pop rax

	ret


; ===================================================================================================
; EOF


;--------------------------------------------------------
; e.g.:
;
; run   ata0 PROGRAMS OS BARENUMBERS file.bin	(command)
; 0     1    2        3  4	     5		(argv)
; 1     2    3        4  5	     6		(argc)
;       0    1        2  3			(level)
;--------------------------------------------------------


;--------------------------------------------------------
; pointers management in save file:
;
; <--          data pointers            ->  <p.p.>   
; --------------------+-------------------+-+---+-+
; |p|p|p|p|...|p|p|p|p|p|p|p|p|...|p|p|p|p|0|p|p|0|
; --------------------+-------------------+-+---+-+
;  *                       *                 *
;  |mem_pages              |mem_pages_off    |mem_pages_p
;



;------------------------------------------------------------------
; erase file:
;					      +-------------------+
;					      V			  |
; HEADER: 	first	(LBA)		last	(LBA) ----------+ |
;								| |
;               +---------------+       +---------------+	| |
; CHAIN:	| N | 0 |	|	| 0 | P |	|	| |
;               +---------------+       +---------------+	| |
;					  ^			| |
;					  |			| |
;		 		    +-----+			| |
;		  		    |				| |
;               +---------------+   |   +---------------+	| |
; Chain: 	| N | 0 |	|   |	| 0 | P |	|	| |
;               +---------------+   |   +---------------+	| |
;		      ^		    |				| |
;		      |		    |				| |
;		      +-------------|---------------------------+ |
;				    |				  |
; File:		first	(LBA) ------+	last	(LBA) ------------+


;----------------------------------------------------------
; Directory Caching:
;
;	Offset
; 01	0	    *  <--- new
; 02	24	... V
; 03	48	...
; 04	72	...
; 05	96	...
; 06	120	When insert new values, shift one position
; 07	144	...
; 08	168	...
; 09	192	 *
; 10	216    * V
; 11	240  * V
; 12	264  V
;
; 12 slots * (64bit*3) => device - lba - memory


;--------------------------------------------------------------------------------------------------------------------------------------
;  (level)		LEVEL 0			LEVEL 1			LEVEL 2			LEVEL 3			file
; 
;  (command)		ata0			SOURCES			OS			BARENUMBERS		main.kernel.asm
; 
; +------------+      +-------------+    
; | Device     | ---- | Root:       |    +---> +--------------+    +---> +--------------+    +---> +--------------+
; +------------+      |-------------|    |     | Directories  |    |     | Directories  |    |     |    Files     | 
;		      | ARCHIVES    | ---+     |     ...      | ---+     |     ...      | ---+     |     ...      | ---> file
; 		      | DATA        |          +--------------+          +--------------+          +--------------+
;		      | DOCUMENTS   |
;		      | IMAGES      |
;		      | MULTIMEDIA  |
;		      | OTHER       |
;		      | PROGRAMS    |
;		      | SOURCES     |
;		      +-------------+
;
;
;
; 		  <-- 8 sectors -->
; +---------------+---------------+---------------+---------------+
; |   LBA 16424   |   LBA 16432   |   LBA 16440   |   LBA 16448   |
; |            	  |   Pointers    |            	  |		  |
; |   HEADER1	  |   ROOT	  |   ROOT	  |   HEADER2	  |
; +---------------+---------------+---------------+---------------+
;
; +---------------+---------------+---------------+---------------+---------------+---------------+---------------+---------------+
; |   LBA 16456   |   LBA 16464   |   LBA 16472   |   LBA 16480   |   LBA 16488   |   LBA 16496   |   LBA 16504   |   LBA 16512   |
; |   Pointers 	  |   Pointers 	  |   Pointers 	  |   Pointers 	  |   Pointers 	  |   Pointers 	  |   Pointers 	  |   Pointers 	  |
; |   ARCHIVES	  |   DATA	  |   DOCUMENTS	  |   IMAGES	  |   MULTIMEDIA  |   OTHER	  |   PROGRAMS	  |   SOURCES	  |
; +---------------+---------------+---------------+---------------+---------------+---------------+---------------+---------------+
;
; +---------------+---------------+---------------+---------------+---------------+---------------+---------------+---------------+
; |   LBA 16520   |   LBA 20584   |   LBA 24648   |   LBA 28712   |   LBA 32776   |   LBA 36840   |   LBA 40904   |   LBA 44968   |
; |            	  |		  |            	  |		  |            	  |		  |            	  |		  |
; |   ARCHIVES	  |   DATA	  |   DOCUMENTS	  |   IMAGES	  |   MULTIMEDIA  |   OTHER	  |   PROGRAMS	  |   SOURCES	  |
; +---------------+---------------+---------------+---------------+---------------+---------------+---------------+---------------+
;
; +---------------+---------------
; |   LBA 49032   |   LBA 49040
; |		  |   		  --->
; |   INIT_FFCC   |   Free
; +---------------+---------------
;
;
;
;
; 		   +-----------------------------------------+
; 		   v +-----------------------------------+   |
; 		     |                                   v   |
;  +------+        +---------------------------------+ +---------------------------------+			  +--------------
;  | file | --->   | N | 0 | * | * | * | * | ... |16 | | N | P | * | * | * | * | ... |16 | ... Pointers Cluster   | 0 | P |
;  +------+        +---------------------------------+ +---------------------------------+			  +--------------
; 		             |				  
; 		             |      +--------------+								  ( Last )
;                            +--->  | Data Cluster | 4096 bytes
; 	         	            +--------------+ 
;
;  +---------------------------------+---+---+ 
;  | N | P | * | * | * | * | ... | * | 0 |16 | ... Pointers Cluster: 8-8-4064-8-8 bytes -> 4064/8 = 508 data clusters -> 2032 KiB addressable
;  +---------------------------------+---+---+	   16 is the offset to first pointer to data
;
;  +--------------+
;  | .		  |  This file :  2032 KiB Max
;  | DIRECTORY 1  |		 16256 items Max
;  | DIRECTORY 2  |		
;  | DIRECTORY 3  |		
;  | DIRECTORY 4  |
;  |    ...	  |
;  |    ...	  |
;  +--------------+




