;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
;
; Keyboard driver     (IRQ1)
;-----------------------------------------------------------------------------------

keyboard:
	push rax
	push rbx
	push rdi
 
	cld
        xor rax, rax
      	in  al, 0x60

	; ESC
	cmp al, 0x01
	je reboot


;====================================================;
; extended key (prefix 0xE0).        		     ;
;====================================================;
;check_e0:
	cmp al, 224
	jne ctrldwn
	mov byte [e0sc_ext], 0xff
	jmp nokey
;====================================================;
; ctrl down                                          ;
;====================================================;
ctrldwn:
       	cmp al, 29		                     
	jne noctrldwn
	mov byte[ctrl_key], 0xff
	jmp nokey
;====================================================;
; ctrl up                                            ;
;====================================================;
noctrldwn:
	cmp al, 29+128		                     
	jne noctrlup
	mov byte[ctrl_key], 0x00
	jmp nokey
;====================================================;
; alt down                                           ;
;====================================================;
noctrlup:
	cmp al, 56		                     
	jne noaltdwn
	mov byte[alt_key], 0xff
	jmp nokey
;====================================================;
; alt up                                             ;
;====================================================;
noaltdwn:
	cmp al, 56+128		                    
	jne noaltup
	mov byte[alt_key], 0x00
	jmp nokey
;====================================================;
; left shift down                                    ;
;====================================================;
noaltup:
	cmp al, 42		                    
	jne nolshftdwn
	mov byte[lshift_key], 0xff
	jmp nokey
;====================================================;
; left shift up                                      ;
;====================================================;
nolshftdwn:
        cmp al, 42+128		                     
	jne nolshftup
	mov byte[lshift_key], 0x00
	jmp nokey
;====================================================;
; right shift down                                   ;
;====================================================;
nolshftup:
	cmp al, 54		                     
        jne norshftdwn
	mov byte[rshift_key], 0xff
	jmp nokey
;====================================================;
; right shift up                                     ;
;====================================================;
norshftdwn:
	cmp al, 54+128		                     
	jne caplockdwn
	mov byte[rshift_key], 0x00
	jmp nokey
;====================================================;
; cap lock down                                      ;
;====================================================;
caplockdwn:
	cmp al, 58		                      
       	jne numlockdwn    
        cmp byte[cap_key], 0xff
        je  @f
	mov byte[cap_key], 0xff
        xor [KeyboardStatus], 4 
        call SetKeyBoardLeds
	jmp nokey
@@:
        mov byte[cap_key], 0x00
        and [KeyboardStatus], 0xfb  ; 1111 1011b
        call SetKeyBoardLeds
	jmp nokey
;====================================================;
; num lock down                                      ;
;====================================================;
numlockdwn:			
        cmp al, 69
        jne scrollockdwn
	cmp byte[numlock_key], 0xff
	je  @f
	mov byte[numlock_key], 0xff
	xor [KeyboardStatus], 2 
        call SetKeyBoardLeds
	jmp nokey
@@:
	mov byte[numlock_key], 0x00
        and [KeyboardStatus], 0xfd  ; 1111 1101b
        call SetKeyBoardLeds
	jmp nokey
;====================================================;
; scroll lock down                                   ;
;====================================================;
scrollockdwn:
	cmp al, 70
	jne numpadconv
	test [KeyboardStatus], 1
	jz @f
	and [KeyboardStatus], 0xfe  ; 1111 1110b
	call SetKeyBoardLeds
	jmp nokey
@@:
        xor [KeyboardStatus], 1
        call SetKeyBoardLeds
	jmp nokey
;====================================================;
; numpad ascii convert                               ;
;====================================================;
numpadconv:
	test [KeyboardStatus], 2
	jz check_e0
	cmp byte[e0sc_ext], 0xff
	je npaddiv

	cmp al, 82
	jne @f
	mov [scancode], al
	mov [key], 48 ; '0'
	jmp nokey
@@:
	cmp al, 79
	jne @f
	mov [scancode], al
	mov [key], 49 ; '1'
	jmp nokey
@@:
	cmp al, 80
	jne @f
	mov [scancode], al
	mov [key], 50 ; '2'
	jmp nokey
@@:
	cmp al, 81
	jne @f
	mov [scancode], al
	mov [key], 51 ; '3'
	jmp nokey
@@:
	cmp al, 75
	jne @f
	mov [scancode], al
	mov [key], 52 ; '4'
	jmp nokey
@@:
	cmp al, 76
	jne @f
	mov [scancode], al
	mov [key], 53 ; '5'
	jmp nokey
@@:
	cmp al, 77
	jne @f
	mov [scancode], al
	mov [key], 54 ; '6'
	jmp nokey
@@:
	cmp al, 71
	jne @f
	mov [scancode], al
	mov [key], 55 ; '7'
	jmp nokey
@@:
	cmp al, 72
	jne @f
	mov [scancode], al
	mov [key], 56 ; '8'
	jmp nokey
@@:
	cmp al, 73
	jne @f
	mov [scancode], al
	mov [key], 57 ; '9'
	jmp nokey
@@:
	cmp al, 55
	jne @f
	mov [scancode], al
	mov [key], 42 ; '*'
	jmp nokey
@@:
	cmp al, 74
	jne @f
	mov [scancode], al
	mov [key], 45 ; '-'
	jmp nokey
@@:
	cmp al, 78
	jne @f
	mov [scancode], al
	mov [key], 43 ; '+'
	jmp nokey
@@:
	cmp al, 83
	jne @f
	mov [scancode], al
	mov [key], 46 ; '.'
	jmp nokey

npaddiv:
	cmp byte[e0sc_ext], 0xff
	jne check_e0
	cmp al, 53
	jne check_e0
	mov [scancode], al
	mov [key], 47 ; '/'
	mov byte[e0sc_ext], 0x00
	jmp nokey

;====================================================;
; Check and set e0sc_ext to zero                     ;
;====================================================;
check_e0:
	cmp byte[e0sc_ext], 0xff
	jne @f
	cmp al, 53			; skip / numpad
	jne @f
	mov byte[e0sc_ext], 0x00
	jmp nokey
@@:
	cmp al, 224			; just in case
	jne @f
	mov byte[e0sc_ext], 0xff
	jmp checkb128
@@: 
	mov byte[e0sc_ext], 0x00
;====================================================;
; Check if below 128, then there is a key.           ;
;====================================================; 
checkb128: 
	cmp al, 128		                      
	jae nokey
	xor rdi, rdi
	mov edi, eax			; offset in keymap table
	cmp byte[rshift_key], 0xff
	je  kshift
	cmp byte[lshift_key], 0xff
	je  kshift
	cmp byte[ctrl_key], 0xff
	je  kctrl
	cmp byte[alt_key], 0xff
	je  kalt
        cmp byte[cap_key], 0xff
        je  kcap
;====================================================;
; Convert to ASCII.                                  ;
;====================================================; 
; normal:
	mov rbx, [normal_keymap]
	add rbx, rdi
	mov ah, byte[rbx]
	jmp keydone
kshift:
	mov rbx, [shift_keymap]
	add rbx, rdi
	mov ah, byte[rbx]
	jmp keydone
kctrl:
	mov rbx, [ctrl_keymap]
	add rbx, rdi
	mov ah, byte[rbx]
	jmp keydone
kalt:
	mov rbx, [alt_keymap]
	add rbx, rdi
	mov ah, byte[rbx]
        jmp keydone      
kcap:
	mov rbx, [caplock_keymap]
	add rbx, rdi   
        mov ah, byte[rbx]
keydone:
        mov [scancode], al
        mov [key], ah
nokey:
	mov al, 0x20			; Acknowledge the IRQ
	out 0x20, al

	call os_smp_wakeup_all		; All

	;mov al, 0			; Wake up only BSP
	;call os_smp_wakeup
	
	pop rdi
	pop rbx
	pop rax
	iretq
; -----------------------------------------------------------------------------

;--------------------;
; Set keyboard leds  ;
;--------------------;
SetKeyBoardLeds:
        push rax
KeyBoardWait1:
	in al, 64h
	test al, 10b			              
	jne KeyBoardWait1
	mov al, 0xed 		                     
	out 60h, al			              
KeyBoardWait2:
	in al, 64h
	test al, 10b			              
	jne KeyBoardWait2	              
	mov al, byte[KeyboardStatus]
	and al, 111b
	out 60h, al			              
        pop rax
	ret

 ;-------------------------------------------------------------------------;
 ; _________________        LED status byte:                               ;
 ; |0|0|0|0|0|1|1|1|                                                       ;
 ; +---------------+                     1 = True   0 = False              ;
 ;            | | +--->  scroll lock                                       ;
 ;            | +----->  num lock                                          ;
 ;            +------->  caps lock                                         ;
 ;-------------------------------------------------------------------------;

 ;-------------------------------
 ; Data                          
 ;-------------------------------

KeyboardStatus  db 0
rshift_key	db 0
lshift_key	db 0
ctrl_key	db 0
alt_key		db 0                 
cap_key         db 0
numlock_key     db 0
e0sc_ext	db 0

normal_keymap	dq 0
shift_keymap	dq 0
ctrl_keymap	dq 0
alt_keymap	dq 0
caplock_keymap	dq 0


;--------------------------------
; keymap special chars
;--------------------------------
_esc 	equ 0
_bspace equ 0
_tab 	equ 0
_enter 	equ 0
_ctrl 	equ 0
_lshift equ 0
_rshift equ 0
_alt 	equ 0
_caps 	equ 0
_f1 	equ 0
_f2 	equ 0
_f3 	equ 0
_f4 	equ 0
_f5 	equ 0
_f6 	equ 0
_f7 	equ 0
_f8 	equ 0
_f9 	equ 0
_f10 	equ 0
_f11 	equ 0
_f12 	equ 0
_prscr	equ 0
_nmlock equ 0
_sclock equ 0
_ins 	equ 0
_home 	equ 0
_del 	equ 0
_end 	equ 0
_pgup 	equ 0
_pgdw 	equ 0
_arup 	equ 0
_ardw 	equ 0
_arlf 	equ 0
_arrg 	equ 0
_lwin	equ 0
_rwin	equ 0
_menu	equ 0


; =============================================================================
; EOF


; SCANCODES KEYBOARD IT(USA):
;
; KEY -  Pushing / Releasing
; -----------------------------
; 	  dec		hex 
; -----------------------------
; a    -  30 158	1e 9e
; b    -  48 176	30 b0
; c    -  46 174	2e ae
; d    -  32 160	20 a0
; e    -  18 146	12 92
; f    -  33 161	21 a1
; g    -  34 162	22 a2
; h    -  35 163	23 a3
; i    -  23 151	17 97
; j    -  36 164	24 a4
; k    -  37 165	25 a5
; l    -  38 166	26 a6
; m    -  50 178	32 b2
; n    -  49 177	31 b1
; o    -  24 152	18 98
; p    -  25 153	19 99
; q    -  16 144	10 90
; r    -  19 147	13 93
; s    -  31 159	1f 9f
; t    -  20 148	14 94
; u    -  22 150	16 96
; v    -  47 175	2f af
; w    -  17 145	11 91
; x    -  45 173	2d ad
; y    -  21 149	15 95
; z    -  44 172	2c ac
; \(`) -  41 169	29 a9
; 1    -   2 130	02 82
; 2    -   3 131	03 83
; 3    -   4 132	04 84
; 4    -   5 133	05 85
; 5    -   6 134	06 86
; 6    -   7 135	07 87
; 7    -   8 136	08 88
; 8    -   9 137	09 89
; 9    -  10 138	0a 8a
; 0    -  11 139	0b 8b
; '(-) -  12 140	0c 8c
; ì(=) -  13 141	0d 8d
; è([) -  26 154	1a 9a
; +(]) -  27 155	1b 9b
; ò(;) -  39 167  	27 a7
; à(') -  40 168  	28 a8
; ù(\) -  43 171	2b ab
; <    -  86 214  	56 d6
; ,    -  51 179	33 b3
; .    -  52 180	34 b4
; -(/) -  53 181	35 b5
; bspace  14 142	0e 8e
; tab     15 143	0f 8f
; enter   28 156	1c 9c
; caps    58 186	3a ba
; lshift  42 170	2a aa
; rshift  54 182	36 b6
; lctrl   29 157	1d 9d
; rctrl 224 29 224 157	e0 1d e0 9d
; alt	  56 184	38 b8
; altgr 224 56 224 184	e0 38 e0 b8
; space   57 185	39 b9
; esc	  01		01
; F1	  59 187	3b bb
; F2	  60 188	3c bc
; F3	  61 189	3d bd
; F4	  62 190	3e be
; F5	  63 191	3f bf
; F6	  64 192	40 c0
; F7	  65 193	41 c1
; F8	  66 194	42 c2
; F9	  67 195	43 c3
; F10	  68 196	44 c4
; F11	  87 215	57 d7
; F12	  88 216	58 d8
;
; NumLock 69 197	45 c5
; ScLock  70 198	46 c6
; Pause	225 29 69 / 225 157 197 - e1 1d 45 / e1 9d c5
; PrtScr  224 42 224 55 / 224 183 224 170 - e0 2a e0 37 / e0 b7 e0 aa
;
; ins  	224 42 224 82 / 224 210 224 170 - e0 2a e0 52 / e0 d2 e0 aa
; home 	224 42 224 71 / 224 199 224 170 - e0 2a e0 47 / e0 c7 e0 aa
; canc 	224 42 224 83 / 224 211 224 170 - e0 2a e0 53 / e0 d3 e0 aa
; end  	224 42 224 79 / 224 207 224 170 - e0 2a e0 4f / e0 cf e0 aa
; pgup 	224 42 224 73 / 224 201 224 170 - e0 2a e0 49 / e0 c9 e0 aa
; pgdw 	224 42 224 81 / 224 209 224 170 - e0 2a e0 51 / e0 d1 e0 aa
; arup 	224 42 224 72 / 224 200 224 170 - e0 2a e0 48 / e0 c8 e0 aa
; ardw 	224 42 224 80 / 224 208 224 170 - e0 2a e0 50 / e0 d0 e0 aa
; arlf 	224 42 224 75 / 224 203 224 170 - e0 2a e0 4b / e0 cb e0 aa
; arrg 	224 42 224 77 / 224 205 224 170 - e0 2a e0 4d / e0 cd e0 aa
;
; Power 224 94 / 224 222 - e0 5e / e0 de
; Sleep 224 95 / 224 223 - e0 5f / e0 df
; Wake  224 99 / 224 227 - e0 63 / e0 e3
;
; NumPad:
; 0	82 210		52 d2
; 1	79 207		4f cf
; 2	80 208		50 d0
; 3	81 209		51 d1
; 4	75 203		4b cb
; 5	76 204		4c cc
; 6	77 205		4d cd
; 7	71 199		47 c7
; 8	72 200		48 c8
; 9	73 201		49 c9
; /  224 53 224 181     e0 35 e0 b5
; *	55 183		37 b7
; -	74 202		4a ca
; +	78 206		4e ce
; .	83 211		53 d3
;
; enter 224 28 224 156    e0 1c e0 9c
;
; ------------------------------------------------------------------
;
; ascii for italian keyb:
; 133 = à		183 = À	(codepage 850)
; 138 = è		212 = È
; 141 = ì		222 = Ì
; 149 = ò		227 = Ò
; 151 = ù		235 = Ù
;
; 130 = é		144 = É
;
; 245 = §
; 248 = °
; 135 = ç
;
;  39 = '

