;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; ATA Identify
;-----------------------------------------------------------------------------------


;--------------------------------------
; IN: 	none regs
; OUT:	none regs
;	[hd_ide_mask]
;	ata info struc(s)
;	descriptor(s)
;--------------------------------------
K_HDD_ATA_Identify:

	push rax
	push rcx
	push r8
	push r9
	push r10

	xor rcx, rcx
	xor r8, r8
	xor r9, r9
	xor r10, r10

	; search drives
	call IDEProbe
	mov [hd_ide_mask], al

	; identify drive
	test [hd_ide_mask], 00000100b		; primary master
	jz .pri_sla_id

	mov r8, ata00_info
	mov r9,  0x01f0
	mov r10, 0x00
	call IDEIdentifyDrive
	cmp eax, 0
	jne .pri_sla_id
	and [hd_ide_mask], 11111011b

.pri_sla_id:
	test [hd_ide_mask], 00001000b		; primary slave
	jz .sec_mas_id	

	mov r8, ata01_info
	mov r9,  0x01f0
	mov r10, 0x10
	call IDEIdentifyDrive
	cmp eax, 0
	jne .sec_mas_id
	and [hd_ide_mask], 11110111b

.sec_mas_id:
	test [hd_ide_mask], 00010000b		; secondary master
	jz .sec_sla_id	

	mov r8, ata10_info
	mov r9,  0x0170
	mov r10, 0x00
	call IDEIdentifyDrive
	cmp eax, 0
	jne .sec_sla_id
	and [hd_ide_mask], 11101111b

.sec_sla_id:
	test [hd_ide_mask], 00100000b		; secondary slave
	jz .pri_mas_device	

	mov r8, ata11_info
	mov r9,  0x0170
	mov r10, 0x10
	call IDEIdentifyDrive
	cmp eax, 0
	jne .pri_mas_device
	and [hd_ide_mask], 11011111b

;--------------------------------------------------------
.pri_mas_device:
	test [hd_ide_mask], 00000100b		; primary master
	jz .pri_sla_device

	; device descriptor
	mov [ata0.present], 1

	; device list
	mov rsi, ata0.string
	mov rdi, device_list
	call os_string_append
	mov rsi, space
	mov rdi, device_list
	call os_string_append

	mov r8, ata00_info
	xor rax, rax
	mov rax, qword [r8 + 200]		; 100*2 -> lba48
	cmp rax, 0
	je .lba28_0
	mov [ata0.readsectors], IDEreadsectorsLBA48
	mov [ata0.writesectors], IDEwritesectorsLBA48
	mov [ata0.tLBA], rax
	jmp .ok0
.lba28_0:
	xor rax, rax
	mov [ata0.readsectors], IDEreadsectorsLBA28
	mov [ata0.writesectors], IDEwritesectorsLBA28
	mov eax, dword [r8 + 120]		; 60*2 -> lba28
	mov [ata0.tLBA], rax
.ok0:
;--------------------------------------------------------

.pri_sla_device:
	test [hd_ide_mask], 00001000b		; primary slave
	jz .sec_mas_device

	; device descriptor
	mov [ata1.present], 1

	; device list
	mov rsi, ata1.string
	mov rdi, device_list
	call os_string_append
	mov rsi, space
	mov rdi, device_list
	call os_string_append

	mov r8, ata01_info
	xor rax, rax
	mov rax, qword [r8 + 200]		; 100*2 -> lba48
	cmp rax, 0
	je .lba28_1
	mov [ata1.readsectors], IDEreadsectorsLBA48
	mov [ata1.writesectors], IDEwritesectorsLBA48
	mov [ata1.tLBA], rax
	jmp .ok1
.lba28_1:
	xor rax, rax
	mov [ata1.readsectors], IDEreadsectorsLBA28
	mov [ata1.writesectors], IDEwritesectorsLBA28
	mov eax, dword [r8 + 120]		; 60*2 -> lba28
	mov [ata1.tLBA], rax
.ok1:
;--------------------------------------------------------

.sec_mas_device:
	test [hd_ide_mask], 00010000b		; secondary master
	jz .sec_sla_device

	; device descriptor
	mov [ata2.present], 1

	; device list
	mov rsi, ata2.string
	mov rdi, device_list
	call os_string_append
	mov rsi, space
	mov rdi, device_list
	call os_string_append

	mov r8, ata10_info
	xor rax, rax
	mov rax, qword [r8 + 200]		; 100*2 -> lba48
	cmp rax, 0
	je .lba28_2
	mov [ata2.readsectors], IDEreadsectorsLBA48
	mov [ata2.writesectors], IDEwritesectorsLBA48
	mov [ata2.tLBA], rax
	jmp .ok2
.lba28_2:
	xor rax, rax
	mov [ata2.readsectors], IDEreadsectorsLBA28
	mov [ata2.writesectors], IDEwritesectorsLBA28
	mov eax, dword [r8 + 120]		; 60*2 -> lba28
	mov [ata2.tLBA], rax
.ok2:
;--------------------------------------------------------

.sec_sla_device:
	test [hd_ide_mask], 00100000b		; secondary slave
	jz .end

	; device descriptor
	mov [ata3.present], 1

	; device list
	mov rsi, ata3.string
	mov rdi, device_list
	call os_string_append
	mov rsi, space
	mov rdi, device_list
	call os_string_append

	mov r8, ata11_info
	xor rax, rax
	mov rax, qword [r8 + 200]		; 100*2 -> lba48
	cmp rax, 0
	je .lba28_3
	mov [ata3.readsectors], IDEreadsectorsLBA48
	mov [ata3.writesectors], IDEwritesectorsLBA48
	mov [ata3.tLBA], rax
	jmp .ok3
.lba28_3:
	xor rax, rax
	mov [ata3.readsectors], IDEreadsectorsLBA28
	mov [ata3.writesectors], IDEwritesectorsLBA28
	mov eax, dword [r8 + 120]		; 60*2 -> lba28
	mov [ata3.tLBA], rax
.ok3:
;--------------------------------------------------------


.end:
	pop r10
	pop r9
	pop r8
	pop rcx
	pop rax

	ret


; ===================================================================================================
; EOF


