;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.2 - GPL3 - Apr 2012
;
;-----------------------------------------------------------------------------------
; System Variables
;-----------------------------------------------------------------------------------


; Memory addresses
;-----------------
buffer64		equ 0x0000000000010000	; 65536 bytes 0x10000 -> 0x1FFFF - 64 KiB Buffers in low memory
os_MemoryMap		equ 0x0000000000300000  ; 131072 bytes	0x300000 -> 0x31FFFF - Location of Memory Map - Room to map 256 GiB with 2 MiB pages
cpustatus		equ 0x0000000000320000	; 256 bytes  0x320000 -> 0x3200FF - Location of CPU status data  Bit 0 = Available, Bit 1 = Free/Busy
cpuqueue		equ 0x0000000000320100	; 4096 bytes 0x320100 -> 0x3210FF - Location of CPU Queue. Each queue item is 16 bytes. (Room for 256 entries)

ata0_header_cache	equ 0x0000000000360000  ; 4096 bytes 0x360000 -> 0x360FFF - Location of header cache    ata0
ata0_root_cache		equ 0x0000000000361000	; 4096 bytes 0x361000 -> 0x361FFF - Location of root cache	ata0
ata1_header_cache	equ 0x0000000000362000  ; 4096 bytes 0x362000 -> 0x362FFF - Location of header cache    ata1
ata1_root_cache		equ 0x0000000000363000	; 4096 bytes 0x363000 -> 0x363FFF - Location of root cache	ata1
ata2_header_cache	equ 0x0000000000364000  ; 4096 bytes 0x364000 -> 0x364FFF - Location of header cache    ata2
ata2_root_cache		equ 0x0000000000365000	; 4096 bytes 0x365000 -> 0x365FFF - Location of root cache	ata2
ata3_header_cache	equ 0x0000000000366000  ; 4096 bytes 0x366000 -> 0x366FFF - Location of header cache    ata3
ata3_root_cache		equ 0x0000000000367000	; 4096 bytes 0x367000 -> 0x367FFF - Location of root cache	ata3

						; 0x368000 -> 0x3EFFFF Reserved for future devices

header_buffer		equ 0x00000000003F0000  ; 4096 bytes 0x3F0000 -> 0x3F0FFF - Location of header buffer
root_buffer		equ 0x00000000003F1000  ; 4096 bytes 0x3F1000 -> 0x3F1FFF - Location of root buffer
cluster_buffer		equ 0x00000000003F2000  ; 4096 bytes 0x3F2000 -> 0x3F2FFF - Location of cluster buffer
cluster_buffer2		equ 0x00000000003F3000  ; 4096 bytes 0x3F3000 -> 0x3F3FFF - Location of cluster buffer2

						; 0x400000 Kernel
programlocation		equ 0x0000000000600000	; Location in memory where programs are loaded


; Variables
;----------

align 16
temp_str: times 21 	db 0			; 20 digit + 1 string terminator
align 16
str_8digit 		db '00000000', 0
align 16
hextable: 		db '0123456789ABCDEF'
align 16
EOF			db  26 ; ascii

align 16
cli_args		db 0
align 16
prg_args		db 0
align 16
cli_command_args_string:	times 1024 db 0
align 16
prg_path_args_string:		times 1024 db 0
align 16
cli_key_com:			times 10240 db 0
align 16
cli_key_com_counter 		db 0

align 16
directory_cache_ptrs:		times 288 db 0	; 24*12 => 64bit=device, 64bit=lba, 64bit=memory (12 pages * 2MiB)

align 16
sector0: 		times 512 db 0
align 16
sector1: 		times 512 db 0
align 16
sector2: 		times 512 db 0

device_list 		db 'Devices list: '	; Identified devices list
device_list_room:	times 256 db 0		; (don't move it)

uuid1 			dq 0	; fs_format.inc
uuid2 			dq 0	; 

os_LocalAPICAddress	dq 0
os_IOAPICAddress	dq 0
os_ClockCounter		dq 0
os_RandomSeed		dq 0	; Seed for RNG
os_Ticks		dq 0
screen_cursor_offset	dq 0


mutex 			dw 1	; The MUTual-EXclustion flag (in cli.inc)
os_NumCores		dw 0
cpuqueuestart		dw 0
cpuqueuefinish		dw 0
os_QueueLen		dw 0
os_QueueLock		dw 0	; Bit 0 clear for unlocked, set for locked.
cpuqueuemax:		dw 256

nscroll			db 0
cursorx			db 0
cursory			db 0
scancode		db 0
key			db 0
key_shift		db 0
screen_cursor_x		db 0
screen_cursor_y		db 0
os_debug_dump_reg_stage	db 0
hd_ide_mask		db 0


;--------------------------------
; Color text attribute
;--------------------------------

ct_black    	equ 0x00	; 0000 0000b
ct_blue     	equ 0x01	; 0000 0001b
ct_green    	equ 0x02	; 0000 0010b
ct_cyan     	equ 0x03	; 0000 0011b
ct_red      	equ 0x04	; 0000 0100b
ct_magenta  	equ 0x05	; 0000 0101b
ct_brown   	equ 0x06	; 0000 0110b
ct_lgray   	equ 0x07	; 0000 0111b

ct_dgray   	equ 0x08	; 0000 1000b
ct_lblue   	equ 0x09	; 0000 1001b
ct_lgreen  	equ 0x0a	; 0000 1010b
ct_lcyan    	equ 0x0b	; 0000 1011b
ct_lred     	equ 0x0c	; 0000 1100b
ct_lmagenta 	equ 0x0d	; 0000 1101b
ct_lbrown   	equ 0x0e	; 0000 1110b
ct_white    	equ 0x0f	; 0000 1111b

ct_yellow   	equ 0x0e    	; useful

ct_whiteonred	equ 01001111b
ct_whiteonblue  equ 00011111b
ct_whiteonbrown equ 01101111b
ct_cyanonred	equ 01000011b
ct_blackonlgray equ 01110000b

;--------------------------------------------------------------
;Bitfields for character's display attribute:
;Bit(s)	Description
; 7	foreground blink or (alternate) background bright
; 6-4	background color 
; 3	foreground bright or (alternate) alternate character set
; 2-0	foreground color 
;---------------------------------------------------------------

; ===================================================================================================
; EOF

